import tkinter as tk
from tkinter import ttk
from tkinter import filedialog


def submit_homework():
    # 获取下拉选择框的值
    submission_method = combo_submission.get()

    # 获取文本输入框的值
    homework_name = entry_name.get()
    working_directory = entry_directory.get()
    command_line = text_command.get("1.0", tk.END)

    # 打印提交的作业信息
    print("提交方式:", submission_method)
    print("作业名称:", homework_name)
    print("工作目录:", working_directory)

    if submission_method == "命令行方式":
        print("命令行:", command_line)

    # 在这里可以将作业提交到服务器或进行其他处理


def select_directory():
    # 弹出选择文件夹对话框
    directory = filedialog.askdirectory()
    # 将选择的文件夹路径显示在输入框中
    entry_directory.delete(0, tk.END)
    entry_directory.insert(tk.END, directory)


# 创建主窗口
window = tk.Tk()
window.title("提交作业")
window.geometry("400x600")

# 创建下拉选择框
label_submission = tk.Label(window, text="提交方式:")
label_submission.pack()
combo_submission = ttk.Combobox(window, values=["命令行方式", "Bash脚本方式", "调度脚本方式"])
combo_submission.pack()

# 创建标签和输入框
label_name = tk.Label(window, text="作业名称:")
label_name.pack()
entry_name = tk.Entry(window)
entry_name.pack()

# 创建选择文件目录按钮
label_directory = tk.Label(window, text="工作目录:")
label_directory.pack()
entry_directory = tk.Entry(window)
entry_directory.pack()
button_select_directory = tk.Button(window, text="选择文件目录", command=select_directory)
button_select_directory.pack()

# 创建命令行方式的文本输入框
label_command = tk.Label(window, text="命令行方式:")
label_command.pack()
text_command = tk.Text(window, height=10)
text_command.pack()

# 创建提交按钮
button_submit = tk.Button(window, text="提交", command=submit_homework)
button_submit.pack()

# 启动主循环
window.mainloop()



# # -*- coding: utf-8 -*-
# import json
# import os
# import pathlib
# import tkinter as tk
# from logging import getLogger
# from pathlib import PurePath, PurePosixPath, PureWindowsPath
# from tkinter import messagebox, ttk
# from tkinter.messagebox import askokcancel, showerror
# from typing import Dict, Iterable, List, Type
#
# from thonny.utils.SugonUtils import get_enable_url, get_tokens, search_files
#
# from thonny import get_runner, get_shell, get_workbench, ui_utils
# from thonny.base_file_browser import (
#     HIDDEN_FILES_OPTION,
#     BaseLocalFileBrowser,
#     BaseRemoteFileBrowser,
#     get_file_handler_conf_key,
# )
# from thonny.common import (
#     IGNORED_FILES_AND_DIRS,
#     InlineCommand,
#     normpath_with_actual_case,
#     universal_dirname,
# )
# from thonny.languages import tr
# from thonny.misc_utils import running_on_windows, sizeof_fmt
# from thonny.running import InlineCommandDialog, construct_cd_command
# from thonny.ui_utils import lookup_style_option
#
# logger = getLogger(__name__)
#
# minsize = 80
#
#
# class FilesView1(tk.PanedWindow):
#     def __init__(self, master=None):
#         tk.PanedWindow.__init__(self, master, orient=tk.HORIZONTAL, borderwidth=0)
#         self.remote_added = False
#
#         self.configure(sashwidth=lookup_style_option("Sash", "sashthickness", 4))
#         self.configure(background=lookup_style_option("TPanedWindow", "background"))
#
#         get_workbench().bind("BackendTerminated", self.on_backend_terminate, True)
#         get_workbench().bind("BackendRestart", self.on_backend_restart, True)
#         get_workbench().bind("WorkbenchClose", self.on_workbench_close, True)
#
#         self.local_files = ActiveLocalFileBrowser(self)
#         self.local_files.check_update_focus()
#         # pane1 = tk.Label(self, text="Pane 1")
#         # # self.add(self.local_files, minsize=minsize)
#         # self.grid(row=0, column=0, sticky="nsew")
#         # self.add(pane1)
#
#         token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
#         enable_url = get_enable_url(token, "efileUrls")
#         search_file = search_files(token, enable_url, 10, 0, None, "")
#         self.data = json.loads(search_file)
#
#         print(self.data, "9999")
#
#         self.tree = ttk.Treeview(self)
#         # 添加根节点
#         self.tree.insert("", "end", text="根目录", open=True)
#
#         # 添加子节点
#         folder1 = self.tree.insert("", "end", text="文件夹1", open=True)
#
#         folder2 = self.tree.insert("", "end", text="文件夹2", open=True)
#         file21 = self.tree.insert(folder2, "end", text="文件2-1")
#         file22 = self.tree.insert(folder2, "end", text="文件2-2")
#         self.tree.insert(file22, "end", text="嘻嘻哈哈或")
#
#         # 绑定点击事件
#         self.tree.bind("<Double-Button-1>", self.toggle_tree)
#         self.add(self.tree)
#
#         # self.remote_files = ActiveRemoteFileBrowser(self)
#         # self.reset_remote()
#
#     def load_files(self, folder_path=None):
#         print(folder_path)
#
#     def toggle_tree(self, event):
#         item_id = self.tree.identify_row(event.y)
#         if item_id:
#             res = self.tree.item(item_id, open=not self.tree.item(item_id, "open"))
#             print(res)
#
#     def on_show(self):
#         self.reset_remote()
#         self.local_files.refresh_tree()
#
#     def reset_remote(self, msg=None):
#         runner = get_runner()
#         if not runner:
#             return
#
#         proxy = runner.get_backend_proxy()
#         if not proxy:
#             self.hide_remote()
#             return
#
#         if proxy.supports_remote_files():
#             # remote pane is needed
#             if not self.remote_added:
#                 self.add(self.remote_files, minsize=minsize)
#                 self.remote_added = True
#                 self.restore_split()
#             self.remote_files.clear()
#             self.remote_files.check_update_focus()
#         else:
#             # remote pane not needed
#             self.hide_remote()
#
#     def hide_remote(self):
#         if self.remote_added:
#             self.save_split()
#             self.remove(self.remote_files)
#             self.remote_added = False
#
#     def save_split(self):
#         _, y = self.sash_coord(0)
#         get_workbench().set_option("view.files_split", y)
#
#     def restore_split(self):
#         split = get_workbench().get_option("view.files_split", None)
#         if split is None:
#             if self.winfo_height() > 5:
#                 split = int(self.winfo_height() * 0.66)
#             else:
#                 split = 600
#
#         self.sash_place(0, 0, split)
#
#     def on_backend_restart(self, event):
#         if event.get("full"):
#             self.reset_remote(event)
#
#     def on_backend_terminate(self, event):
#         self.reset_remote(event)
#
#     def on_workbench_close(self, event=None):
#         if self.remote_added:
#             self.save_split()
#
#     def get_active_local_dir(self):
#         return self.local_files.get_active_directory()
#
#     def get_active_remote_dir(self):
#         if self.remote_added:
#             return self.remote_files.get_active_directory()
#         else:
#             return None
#
#     def destroy(self):
#         get_workbench().unbind("BackendTerminated", self.on_backend_terminate)
#         get_workbench().unbind("BackendRestart", self.on_backend_restart)
#         get_workbench().unbind("WorkbenchClose", self.on_workbench_close)
#         super().destroy()
#
#
# class ActiveLocalFileBrowser(BaseLocalFileBrowser):
#     def __init__(self, master):
#         super().__init__(master)
#         get_workbench().bind("ToplevelResponse", self.on_toplevel_response, True)
#
#     def is_active_browser(self):
#         return True
#
#     def create_new_file(self):
#         path = super().create_new_file()
#         if path and path.endswith(".py"):
#             get_workbench().get_editor_notebook().show_file(path)
#
#     def get_proposed_new_file_name(self, folder, extension):
#         base = "new_file"
#
#         if os.path.exists(os.path.join(folder, base + extension)):
#             i = 2
#
#             while True:
#                 name = base + "_" + str(i) + extension
#                 path = os.path.join(folder, name)
#                 if os.path.exists(path):
#                     i += 1
#                 else:
#                     return name
#         else:
#             return base + extension
#
#     def request_focus_into(self, path):
#         if path == "":
#             if running_on_windows():
#                 # list of drives, can't cd
#                 self.focus_into(path)
#                 return
#             else:
#                 path = "/"
#
#         if not os.path.isdir(path):
#             return
#
#         proxy = get_runner().get_backend_proxy()
#         if (
#             proxy
#             and proxy.uses_local_filesystem()
#             and proxy.get_cwd() != path
#             and get_runner().is_waiting_toplevel_command()
#         ):
#             get_shell().submit_magic_command(construct_cd_command(normpath_with_actual_case(path)))
#         else:
#             # it's OK, if it's already focused into this directory
#             # focus again to refresh
#             self.focus_into(path)
#             get_workbench().set_local_cwd(path)
#
#     def on_toplevel_response(self, event):
#         self.check_update_focus()
#
#     def check_update_focus(self):
#         cwd = get_workbench().get_local_cwd()
#         if cwd != self.current_focus and os.path.isdir(cwd):
#             self.focus_into(cwd)
#
#     def check_add_upload_command(self):
#         target_dir = self.master.get_active_remote_dir()
#         if target_dir is None:
#             return
#
#         proxy = get_runner().get_backend_proxy()
#
#         if not proxy.supports_remote_directories():
#             target_dir_desc = proxy.get_node_label()
#         else:
#             target_dir_desc = target_dir
#
#         def _upload():
#             selection = self.get_selection_info(True)
#             if not selection:
#                 return
#
#             if "dir" in selection["kinds"] and not proxy.supports_remote_directories():
#                 messagebox.showerror(
#                     "Can't upload directory",
#                     "%s does not support directories.\n" % proxy.get_node_label()
#                     + "You can only upload files.",
#                     master=self,
#                 )
#             else:
#                 if upload(selection["paths"], target_dir, master=self):
#                     self.master.remote_files.refresh_tree()
#
#         self.menu.add_command(label=tr("Upload to %s") % target_dir_desc, command=_upload)
#
#     def add_first_menu_items(self, context):
#         if self.check_for_venv():
#             self.menu.add_command(
#                 label=tr("Activate virtual environment"), command=lambda: self.do_activate_venv()
#             )
#             self.menu.add_separator()
#
#         super().add_first_menu_items(context)
#
#     def add_middle_menu_items(self, context):
#         self.check_add_upload_command()
#         super().add_middle_menu_items(context)
#
#     def _get_venv_path(self):
#         path = self.get_selected_path()
#         if not path:
#             return None
#
#         CFGFILE = "pyvenv.cfg"
#         fnam = self.get_selected_name()
#         try:
#             if os.path.exists(path):
#                 if os.path.isdir(path):
#                     cfgfile = os.path.join(path, CFGFILE)
#                     if os.path.exists(cfgfile) and os.path.isfile(cfgfile):
#                         return path
#                 else:
#                     if fnam == CFGFILE:
#                         return os.path.dirname(path)
#         except Exception:
#             import traceback
#
#             traceback.print_stack()
#             logger.exception("_get_venv_path")
#
#     def check_for_venv(self):
#         return self._get_venv_path() is not None
#
#     def do_activate_venv(self):
#         venv_path = self._get_venv_path()
#
#         if running_on_windows():
#             backend_python = os.path.join(venv_path, "Scripts", "python.exe")
#         else:
#             backend_python = os.path.join(venv_path, "bin", "python3")
#
#         if os.path.isfile(backend_python):
#             get_workbench().set_option("run.backend_name", "LocalCPython")
#             get_workbench().set_option("LocalCPython.executable", backend_python)
#
#             # just like pressing the button
#             get_runner().cmd_stop_restart()
#         else:
#             messagebox.showerror("Error", f"Could not find {backend_python!r}", master=self)
#
#
# class ActiveRemoteFileBrowser(BaseRemoteFileBrowser):
#     def __init__(self, master):
#         super().__init__(master)
#         get_workbench().bind("ToplevelResponse", self.on_toplevel_response, True)
#         get_workbench().bind("RemoteFilesChanged", self.on_remote_files_changed, True)
#
#     def is_active_browser(self):
#         return True
#
#     def supports_new_file(self):
#         return True
#
#     def on_toplevel_response(self, msg):
#         if not self.winfo_ismapped():
#             return
#         if get_runner().get_backend_proxy().supports_remote_files():
#             # pass cwd, as proxy may not yet know it
#             self.check_update_focus(msg.get("cwd"))
#
#     def on_remote_files_changed(self, event=None):
#         if not self.winfo_ismapped():
#             return
#
#         if get_runner().get_backend_proxy().supports_remote_files():
#             self.refresh_tree()
#
#     def check_update_focus(self, new_cwd=None):
#         if new_cwd is None:
#             proxy = get_runner().get_backend_proxy()
#             new_cwd = proxy.get_cwd()
#
#         if self.current_focus != new_cwd:
#             self.focus_into(new_cwd)
#
#     def request_new_focus(self, path):
#         get_shell().submit_magic_command(["%cd", path if path != "" else "/"])
#
#     def add_download_command(self):
#         target_dir = self.master.get_active_local_dir()
#
#         def download():
#             selection = self.get_selection_info(True)
#             if not selection:
#                 return
#
#             dlg = DownloadDialog(
#                 self,
#                 selection["paths"],
#                 selection["description"],
#                 target_dir,
#             )
#             ui_utils.show_dialog(dlg)
#             if dlg.response is not None:
#                 self.master.local_files.refresh_tree()
#
#         self.menu.add_command(label=tr("Download to %s") % target_dir, command=download)
#
#     def add_middle_menu_items(self, context):
#         self.add_download_command()
#         super().add_middle_menu_items(context)
#
#
# class TransferDialog(InlineCommandDialog):
#     def _on_response(self, response):
#         if response.get("command_id") != self._cmd["id"]:
#             return
#
#         if self._stage == "preparation":
#             if self._confirm_and_start_main_work(response):
#                 self._stage = "main_work"
#             else:
#                 self.response = None
#                 self.report_done(True)
#
#         elif self._stage == "main_work":
#             self.response = response
#             self.report_done(self._check_success(response))
#
#         self.update_ui()
#
#     def _confirm_and_start_main_work(self, response):
#         raise NotImplementedError()
#
#     def _check_success(self, response):
#         if response.get("error"):
#             self.set_action_text("Error")
#             self.append_text("\nError: %s\n" % response["error"])
#             return False
#         elif response["errors"]:
#             self.set_action_text("Error")
#             self.append_text("\nError: %s\n" % format_items(response["errors"]))
#             return False
#         else:
#             self.set_action_text("Done!")
#             return True
#
#
# class UploadDialog(TransferDialog):
#     def __init__(self, master, paths, target_dir):
#         self._stage = "preparation"
#         self.items = []
#         source_names = []
#         for path in paths:
#             source_context_dir = os.path.dirname(path)
#             for item in prepare_upload_items(path, source_context_dir, target_dir):
#                 # same path could have been provided directly and also via its parent
#                 if item not in self.items:
#                     self.items.append(item)
#                     source_names.append(os.path.basename(item["source_path"]))
#
#         target_paths = [x["target_path"] for x in self.items]
#         cmd = InlineCommand(
#             "prepare_upload",
#             target_paths=target_paths,
#             description=get_transfer_description("Uploading", source_names, target_dir),
#         )
#
#         super(UploadDialog, self).__init__(master, cmd, "Uploading")
#
#     def _confirm_and_start_main_work(self, preparation_response):
#         picked_items = list(
#             sorted(
#                 pick_transfer_items(self.items, preparation_response["existing_items"], self),
#                 key=lambda x: x["target_path"],
#             )
#         )
#         if picked_items:
#             backend_name = get_runner().get_backend_proxy().get_backend_name()
#             self._cmd = InlineCommand(
#                 "upload",
#                 items=picked_items,
#                 make_shebang_scripts_executable=get_workbench().get_option(
#                     f"{backend_name}.make_uploaded_shebang_scripts_executable"
#                 ),
#             )
#             get_runner().send_command(self._cmd)
#             return True
#         else:
#             return False
#
#
# class DownloadDialog(TransferDialog):
#     def __init__(self, master, paths, description, target_dir):
#         self._stage = "preparation"
#         self._target_dir = target_dir
#
#         cmd = InlineCommand(
#             "prepare_download",
#             source_paths=paths,
#             description=tr("Downloading %s to %s") % (description, target_dir),
#         )
#
#         super(DownloadDialog, self).__init__(master, cmd, "Downloading")
#
#     def _prepare_download_items(
#         self, all_source_items: Dict[str, Dict], target_dir: str
#     ) -> List[Dict]:
#         result = []
#         for source_path, source_item in all_source_items.items():
#             source_context_dir = universal_dirname(source_item["anchor"])
#             result.append(
#                 {
#                     "kind": source_item["kind"],
#                     "size": source_item["size"],
#                     "source_path": source_path,
#                     "target_path": transpose_path(
#                         source_path, source_context_dir, target_dir, PurePosixPath, pathlib.Path
#                     ),
#                 }
#             )
#         return result
#
#     def _get_existing_target_items(self, prepared_items: List[Dict]) -> Dict[str, Dict]:
#         result = {}
#
#         for item in prepared_items:
#             target_path = item["target_path"]
#             if os.path.exists(target_path):
#                 if os.path.isdir(target_path):
#                     kind = "dir"
#                     size = None
#                 else:
#                     kind = "file"
#                     size = os.path.getsize(target_path)
#
#                 result[target_path] = {
#                     "kind": kind,
#                     "size": size,
#                 }
#         return result
#
#     def _confirm_and_start_main_work(self, preparation_response):
#         prepared_items = self._prepare_download_items(
#             preparation_response["all_items"], self._target_dir
#         )
#         existing_target_items = self._get_existing_target_items(prepared_items)
#         picked_items = pick_transfer_items(prepared_items, existing_target_items, self)
#         if picked_items:
#             self._cmd = InlineCommand("download", items=picked_items)
#             get_runner().send_command(self._cmd)
#             return True
#         else:
#             return False
#
#
# def transpose_path(
#     source_path: str,
#     source_dir: str,
#     target_dir: str,
#     source_path_class: Type[PurePath],
#     target_path_class: Type[PurePath],
# ) -> str:
#     assert not source_dir.endswith(":")
#     source_path_parts = source_path_class(source_path).parts
#     source_dir_parts = source_path_class(source_dir).parts
#     assert source_path_parts[: len(source_dir_parts)] == source_dir_parts
#     source_suffix_parts = source_path_parts[len(source_dir_parts) :]
#
#     target = target_path_class(target_dir).joinpath(*source_suffix_parts)
#     return str(target)
#
#
# def pick_transfer_items(
#     prepared_items: List[Dict], existing_target_items: Dict[str, Dict], master
# ) -> List[Dict]:
#     if not existing_target_items:
#         return prepared_items
#
#     errors = []
#     overwrites = []
#
#     for item in prepared_items:
#         if item["target_path"] in existing_target_items:
#             target_info = existing_target_items[item["target_path"]]
#             if item["kind"] != target_info["kind"]:
#                 errors.append(
#                     "Can't overwrite '%s' with '%s', because former is a %s but latter is a %s"
#                     % (item["target_path"], item["source_path"], target_info["kind"], item["kind"])
#                 )
#             elif item["kind"] == "file":
#                 size_diff = item["size"] - target_info["size"]
#                 if size_diff > 0:
#                     replacement = "a larger file (%s + %s)" % (
#                         sizeof_fmt(target_info["size"]),
#                         sizeof_fmt(size_diff),
#                     )
#                 elif size_diff < 0:
#                     replacement = "a smaller file (%s - %s)" % (
#                         sizeof_fmt(target_info["size"]),
#                         sizeof_fmt(-size_diff),
#                     )
#                 else:
#                     replacement = "a file of same size (%s)" % sizeof_fmt(target_info["size"])
#
#                 overwrites.append("'%s' with %s" % (item["target_path"], replacement))
#
#     if errors:
#         showerror("Error", format_items(errors), master=master)
#         return []
#     elif overwrites:
#         if askokcancel(
#             "Overwrite?",
#             "This operation will overwrite\n\n" + format_items(overwrites),
#             master=master,
#         ):
#             return prepared_items
#         else:
#             return []
#     else:
#         return prepared_items
#
#
# def format_items(items):
#     max_count = 10
#     if len(items) == 1:
#         return items[0]
#     msg = "• " + "\n• ".join(items[:max_count])
#     if len(items) > max_count:
#         msg += "\n ... %d more ..." % (len(items) - max_count)
#
#     return msg
#
#
# def upload(paths, target_dir, master) -> bool:
#     dlg = UploadDialog(master, paths, target_dir)
#     ui_utils.show_dialog(dlg)
#     return dlg.response is not None
#
#
# def prepare_upload_items(
#     source_path: str, source_context_dir: str, target_dir: str
# ) -> Iterable[Dict]:
#     # assuming target system has Posix paths
#     if os.path.isdir(source_path):
#         kind = "dir"
#         size = None
#     else:
#         kind = "file"
#         size = os.path.getsize(source_path)
#
#     result = [
#         {
#             "kind": kind,
#             "size": size,
#             "source_path": source_path,
#             "target_path": transpose_path(
#                 source_path, source_context_dir, target_dir, pathlib.Path, PurePosixPath
#             ),
#         }
#     ]
#
#     if os.path.isdir(source_path):
#         for child in os.listdir(source_path):
#             if child not in IGNORED_FILES_AND_DIRS:
#                 result.extend(
#                     prepare_upload_items(
#                         os.path.join(source_path, child), source_context_dir, target_dir
#                     )
#                 )
#     return result
#
#
# def get_transfer_description(verb, paths, target_dir):
#     if len(paths) == 1:
#         subject = "'%s'" % paths[0]
#     else:
#         subject = "%d items" % len(paths)
#
#     return "%s %s to %s" % (verb, subject, target_dir)
#
#
# def load_plugin() -> None:
#     get_workbench().set_default(
#         "file.last_browser_folder", normpath_with_actual_case(os.path.expanduser("~"))
#     )
#
#     get_workbench().set_default(HIDDEN_FILES_OPTION, False)
#
#     get_workbench().add_view(FilesView1, tr("Files_yb"), "nw")
#
#     for ext in [
#         ".py",
#         ".pyw",
#         ".pyi",
#         ".txt",
#         ".log",
#         ".json",
#         ".yml",
#         ".yaml",
#         ".md",
#         ".rst",
#         ".toml",
#         ".gitignore",
#         ".env",
#     ]:
#         get_workbench().set_default(get_file_handler_conf_key(ext), "thonny")
#
#
# if __name__ == "__main__":
#     print(
#         transpose_path(r"C:\kala\pala\kama.py", "C:", "/home/aivar", PureWindowsPath, PurePosixPath)
#     )



# -*- coding: utf-8 -*-

import datetime
import platform
import sys
import tkinter as tk
import tkinter.font
from logging import getLogger
from tkinter import messagebox, ttk

from thonny.utils.internet_utils import InternetUtil

import thonny
from thonny import get_workbench, ui_utils
from thonny.common import get_python_version_string
from thonny.languages import tr
from thonny.mysql_sql import (
    IAnswerService,
    IExamService,
    IHomeworkService,
    IQuestionService,
    IQuestionStemService,
    IStemService,
    IUpdateVersion,
    IVideoService,
)
from thonny.sqlite3_sql import (
    AnswerService,
    ExamService,
    HomeworkService,
    QuestionService,
    QuestionStemService,
    StemService,
    UserCourseService,
    UserService,
    VideoService,
)
from thonny.ui_utils import CommonDialogEx, create_url_label

logger = getLogger(__name__)


# 定义了一个AboutDialog类，继承自CommonDialogEx类，
# 用于创建关于Thonny的对话框。对话框中包含了Thonny的版本号、官方网站链接、开发者信息、许可信息等内容。
class AboutDialog(CommonDialogEx):
    def __init__(self, master):
        super().__init__(master)

        self.title(tr("About Thonny"))  # 设置对话框标题
        self.resizable(height=tk.FALSE, width=tk.FALSE)  # 禁止调整对话框大小

        default_heading_font = tkinter.font.nametofont("TkHeadingFont")
        heading_font = default_heading_font.copy()
        heading_font.configure(size=int(default_heading_font["size"] * 1.7), weight="bold")
        heading_label = ttk.Label(
            self.main_frame, text="Thonny " + thonny.get_version(), font=heading_font
        )
        heading_label.grid(pady=(self.get_large_padding(), self.get_small_padding()))

        url_label = create_url_label(self.main_frame, "https://thonny.org", justify=tk.CENTER)
        url_label.grid()

        if sys.platform == "linux":
            try:
                import distro  # 不需要安装distro模块

                system_desc = distro.name(True)
            except ImportError:
                system_desc = "Linux"

            if "32" not in system_desc and "64" not in system_desc:
                system_desc += self.get_os_word_size_suffix()
        elif sys.platform == "darwin":
            mac_ver = platform.mac_ver()[0]
            mac_arch = platform.mac_ver()[2]
            system_desc = f"macOS {mac_ver} ({mac_arch})"
        else:
            release = platform.release()
            if sys.platform == "win32":
                # Win 10 and 11 both give 10 as release
                try:
                    build = int(platform.version().split(".")[2])
                    if release == "10" and build >= 22000:
                        release = "11"
                except Exception:
                    logger.exception("Could not determine Windows version")

            system_desc = platform.system() + " " + release + self.get_os_word_size_suffix()

        platform_label = ttk.Label(
            self.main_frame,
            justify=tk.CENTER,
            text=system_desc
            + "\n"
            + "Python "
            + get_python_version_string()
            + "\n"
            + "Tk "
            + ui_utils.get_tk_version_str(),
        )
        platform_label.grid(pady=self.get_medium_padding())

        credits_label = create_url_label(
            self.main_frame,
            "https://github.com/thonny/thonny/blob/master/CREDITS.rst",
            tr(
                "Made in\n"
                + "University of Tartu, Estonia,\n"
                + "with the help from\n"
                + "open-source community,\n"
                + "Raspberry Pi Foundation\n"
                + "and Cybernetica AS"
            ),
            justify=tk.CENTER,
        )
        credits_label.grid()

        default_font = tkinter.font.nametofont("TkDefaultFont")
        license_font = default_font.copy()
        license_font.configure(size=round(default_font["size"] * 0.7))
        license_label = ttk.Label(
            self.main_frame,
            text="Copyright (©) "
            + str(datetime.datetime.now().year)
            + " Aivar Annamaa\n"
            + tr(
                "This program comes with\n"
                + "ABSOLUTELY NO WARRANTY!\n"
                + "It is free software, and you are welcome to\n"
                + "redistribute it under certain conditions, see\n"
                + "https://opensource.org/licenses/MIT\n"
                + "for details"
            ),
            justify=tk.CENTER,
            font=license_font,
        )
        license_label.grid(pady=self.get_medium_padding())

        ok_button = ttk.Button(
            self.main_frame, text=tr("OK"), command=self.on_close, default="active"
        )
        ok_button.grid(pady=(0, self.get_large_padding()))
        ok_button.focus_set()

        self.bind("<Return>", self.on_close, True)

    def get_os_word_size_suffix(self):
        if "32" in platform.machine() and "64" not in platform.machine():
            return " (32-bit)"
        else:
            return ""


# load_plugin函数用于加载插件
def load_plugin() -> None:
    def _check_version_and_update():
        # 检查网络状况
        internet_util = InternetUtil()
        connection = internet_util.check_internet_connection()
        if not connection:
            messagebox.showerror("系统提示", "版本及数据更新失败，检测到未连接网络，请确认网络状态")
        else:
            print("版本更新")
            # 版本及数据对比
            # 对比表中每一个id相同的远程数据版本号和本地版本号是否一致，不一致，单独更新该条数据
            compare_data_count()

    def compare_data_count():
        user_service = UserService()
        user = user_service.get_one()
        # 用户信息
        user_type = user[3]
        user_course_service = UserCourseService()
        user_course_list = user_course_service.list_user_course()
        print(user_course_list)
        # 用户课程
        course_array = []
        for i, user_course in enumerate(user_course_list):
            course_array.insert(i, user_course[2])

        # 查询哪些表需要更新
        i_update_version = IUpdateVersion()
        update_versions = i_update_version.check_update_version(user_type, course_array)
        for update_version in update_versions:
            table_name_ = update_version["table_name"]
            course_type = update_version["course_type"]
            if table_name_ == thonny.mysql_sql.video_table:
                # 获取远程视频数据
                i_video_service = IVideoService()
                videos = i_video_service.list_video(user_type, course_type)
                # 比较本地视频数据并进行更新
                video_service = VideoService()
                video_service.origin_update_video(videos)
            elif table_name_ == thonny.mysql_sql.stem_table:
                # 获取远程题目数据
                i_stem_service = IStemService()
                stems = i_stem_service.list_stem(user_type, course_type)
                # 比较本地题目数据并进行更新
                stem_service = StemService()
                stem_service.origin_update_stem(stems)
            elif table_name_ == thonny.mysql_sql.answer_table:
                # 获取远程答案数据
                i_answer_service = IAnswerService()
                answers = i_answer_service.list_answer(user_type, course_type)
                # 比较本地答案数据并进行更新
                answer_service = AnswerService()
                answer_service.origin_update_answer(answers)
            elif table_name_ == thonny.mysql_sql.question_table:
                # 获取远程题组数据
                i_question_service = IQuestionService()
                questions = i_question_service.list_question(user_type, course_type)
                # 比较本地题组数据并进行更新
                question_service = QuestionService()
                question_service.origin_update_question(questions)
            elif table_name_ == thonny.mysql_sql.question_stem_table:
                # 获取远程题目题组数据
                i_question_stem_service = IQuestionStemService()
                question_stems = i_question_stem_service.list_question_stem(user_type, course_type)
                # 比较本地题目题组数据并进行更新
                question_stem_service = QuestionStemService()
                question_stem_service.origin_update_question_stem(question_stems)
            elif table_name_ == thonny.mysql_sql.homework_table:
                # 获取远程作业数据
                i_homework_service = IHomeworkService()
                homeworks = i_homework_service.list_homework(user_type, course_type)
                # 比较本地作业数据并进行更新
                homework_service = HomeworkService()
                homework_service.origin_update_homework(homeworks)
            elif table_name_ == thonny.mysql_sql.exam_table:
                # 获取远程考试数据
                i_exam_service = IExamService()
                exams = i_exam_service.list_exam(user_type, course_type)
                # 比较本地考试数据并进行更新
                exam_service = ExamService()
                exam_service.origin_update_exam(exams)

    def open_about():
        # 打开对话框
        ui_utils.show_dialog(AboutDialog(get_workbench()))

    def open_url(url):
        import webbrowser

        # webbrowser.open返回bool值，但add_command需要None值
        webbrowser.open(url)

    # 添加到下拉菜单help里  绑定命令：打开版本历史记录的链接
    get_workbench().add_command(
        "changelog",
        "help",
        tr("版本历史"),
        lambda: open_url("https://github.com/thonny/thonny/blob/master/CHANGELOG.rst"),
        group=60,
    )
    # 添加到下拉菜单help里 绑定命令：打开报告问题的链接
    get_workbench().add_command(
        "issues",
        "help",
        tr("报告问题"),
        lambda: open_url("https://github.com/thonny/thonny/issues"),
        group=60,
    )

    # 添加到下拉菜单help里 绑定命令：检查版本及数据更新
    get_workbench().add_command(
        "version",
        "help",
        tr("版本检查"),
        _check_version_and_update,
        group=60,
    )

    # 添加到下拉菜单help里：打开关于Thonny的对话框
    get_workbench().add_command("about", "help", tr("关于Thonny"), open_about, group=61)

    # For Mac
    get_workbench().createcommand("tkAboutDialog", open_about)



import sqlite3
import tkinter as tk
from tkinter import ttk

from thonny.exam import ExamListApp
from thonny.sqlite3_sql import ExamService, HomeworkService
from thonny.ui_utils import CommonDialog
from thonny.vendored_libs.score import scoreListApp


class homeworkAndExamDialog(CommonDialog):
    def __init__(self, master):
        super().__init__(master=master)
        ex_w = self.winfo_screenwidth()
        ex_h = self.winfo_screenheight()
        self.title("作业与考试")
        self.geometry("800x800+" + str(int(ex_w / 2) - 400) + "+" + str(int(ex_h / 2) - 400))

        self.protocol("WM_DELETE_WINDOW", self.on_closing)
        self.title_name = "我的作业"
        self.table_name = "t_homework"

        # 创建选项卡控件
        self.tabControl = ttk.Notebook(self)
        self.tabControl.pack(fill="both", expand=1)

        # 创建作业选项卡
        self.homework_tab = ttk.Frame(self.tabControl)
        self.tabControl.add(self.homework_tab, text="作业")
        header_labels = ["作业序号", "作业名称", "作答时间（秒）", "作业简介"]
        for i, label in enumerate(header_labels):
            header_label = tk.Label(self.homework_tab, text=label, font=("bold", 12))
            header_label.grid(row=0, column=i, padx=10, pady=5, ipady=20, sticky="nsew")
            self.homework_tab.columnconfigure(1, weight=1)

        homework = HomeworkService()
        list1 = homework.list_homework()
        homework_list = []
        for item in list1:
            print("homework_list", item)
            homework_list.append(
                {
                    "id": item[0],
                    "name": item[1],
                    "time": item[4],
                    "description": item[6],
                    "question_id": item[7],
                }
            )
        print("所有作业", homework_list)

        ExamListApp(self.homework_tab, exam_list=homework_list, type="开始答题")

        # 创建考试选项卡
        self.exam_tab = ttk.Frame(self.tabControl)
        self.tabControl.add(self.exam_tab, text="考试")

        header_labels = ["考试序号", "考试名称", "作答时间（秒）", "考试简介"]
        for i, label in enumerate(header_labels):
            header_label = tk.Label(self.exam_tab, text=label, font=("bold", 12))
            header_label.grid(row=0, column=i, padx=10, pady=5, ipady=30, sticky="nsew")
            self.exam_tab.columnconfigure(1, weight=1)

        exam_list = []
        exam = ExamService()
        list2 = exam.list_exam()
        for item in list2:
            exam_list.append(
                {
                    "id": item[0],
                    "name": item[1],
                    "time": item[4],
                    "description": item[6],
                    "question_id": item[7],
                }
            )

        ExamListApp(self.exam_tab, exam_list=exam_list, type="开始考试")

        # 创建作业成绩查询选项卡
        self.homework_score_tab = ttk.Frame(self.tabControl)
        self.tabControl.add(self.homework_score_tab, text="作业成绩")

        header_labels = ["作业序号", "作业名称", "分数", "作答次数"]
        tk.Button(self.homework_score_tab, text="刷新", command=self.refreshWork).grid(row=0, pady=10)
        for i, label in enumerate(header_labels):
            header_label = tk.Label(self.homework_score_tab, text=label, font=("bold", 12))
            header_label.grid(row=1, column=i, padx=10, pady=5, ipady=30, sticky="nsew")
            self.homework_score_tab.columnconfigure(1, weight=1)

        homework_score_list = []
        homework_score = homework.list_homework_score()
        for item in homework_score:
            homework_score_list.append(
                {"id": item[0], "name": item[1], "score": item[5], "submit_count": item[6]}
            )

        scoreListApp(self.homework_score_tab, score_list=homework_score_list, type="查看详情")

        # 创建考试成绩查询选项卡
        self.exam_score_tab = ttk.Frame(self.tabControl)
        self.tabControl.add(self.exam_score_tab, text="考试成绩")

        header_labels = ["作业序号", "作业名称", "分数", "作答次数"]
        tk.Button(self.exam_score_tab, text="刷新", command=self.refreshExam).grid(row=0, pady=10)
        for i, label in enumerate(header_labels):
            header_label = tk.Label(self.exam_score_tab, text=label, font=("bold", 12))
            header_label.grid(row=1, column=i, padx=10, pady=5, ipady=30, sticky="nsew")
            self.exam_score_tab.columnconfigure(1, weight=1)

        exam_score_list = []
        exam_score = exam.list_exam_score()
        for item in exam_score:
            exam_score_list.append(
                {"id": item[0], "name": item[1], "score": item[5], "submit_count": item[6]}
            )

        scoreListApp(self.exam_score_tab, score_list=exam_score_list, type="查看详情")

        # 为选项卡绑定事件
        self.tabControl.bind("<<NotebookTabChanged>>", self.on_tab_change)

    def on_closing(self):
        self.destroy()  # 销毁窗口
    def refreshExam(self):
        exam = ExamService()
        exam_score_list = []
        exam_score = exam.list_exam_score()
        for item in exam_score:
            exam_score_list.append(
                {"id": item[0], "name": item[1], "score": item[5], "submit_count": item[6]}
            )
        scoreListApp(self.exam_score_tab, score_list=exam_score_list, type="查看详情")
    def refreshWork(self):
        homework = HomeworkService()
        homework_score_list = []
        homework_score = homework.list_homework_score()
        for item in homework_score:
            homework_score_list.append(
                {"id": item[0], "name": item[1], "score": item[5], "submit_count": item[6]}
            )
        scoreListApp(self.homework_score_tab, score_list=homework_score_list, type="查看详情")

    def on_tab_change(self, event):
        # print('我执行了')
        selected_tab = event.widget.select()
        tab_index = event.widget.index(selected_tab)

        if tab_index == 0:  # 作业选项卡
            self.show_data("HOMEWORK")
        elif tab_index == 1:  # 考试选项卡
            self.show_data("EXAM")

    def show_data(self, data_type):
        homework_service = HomeworkService()
        if data_type == "EXAM":
            self.table_name = "t_exam"
            self.title_name = "我的考试"
        data = homework_service.get_data(self.table_name)
        # self.show_tree_window(data)

    def show_tree_window(self, data):
        print(data)
        tree_window = tk.Toplevel(self)
        tree_window.title(self.title_name)
        tree_window.grab_set()
        tree = ttk.Treeview(tree_window)
        tree.heading("#0", text="名称")
        tree.pack()

        for item in data:
            tree.insert("", "end", text=item[0])



import os
import re
import tkinter as tk
from io import BytesIO
import sys
from tkinter import messagebox, ttk

import requests
from PIL import Image, ImageTk

from thonny.sqlite3_sql import UserService

# # 账号
# account = None
# # 密码
# password = None
# # 全局窗口
# root = None


class UserLogin:
    def __init__(self):
        self.account = None
        self.password = None
        self.login_image = None
        self.root = tk.Tk()
        self.login_result = False
        self.canvas = None
        self.bg_img = None
        self.account_var = None
        self.account_entry = None

        # 添加"密码："标签和输入框
        self.password_var = None
        self.password_entry = None
        self.root.bind("<KeyPress-Return>", lambda event: self.login())  # 绑定回车键

        self.root.protocol("WM_DELETE_WINDOW", self.null_def)
    def login(self):
        userService = UserService()
        account = self.account_var.get()
        password = self.password_var.get()

        # 校验账号和密码
        if not account:
            messagebox.showerror("登录结果", "账号不能为空")
            return
            # 校验手机号码
        pattern = r'^1[3457896]\d{9}$'
        if not re.match(pattern, account):
            messagebox.showerror(title='This is the title', message="电话号码错误", detail='请输入有效的电话号码', icon=messagebox.ERROR)
            # for child in self.fr.winfo_children():
            #     child.configure(state="normal")
            self.account_entry.config(state="normal")  # 设置账号输入框为可编辑状态
            self.password_entry.config(state="normal")  # 设置密码输入框为可编辑状态

            return

        if not password:
            messagebox.showerror("登录结果", "密码不能为空")
            return

        if len(password) < 6:
            messagebox.showerror("登录结果", "密码不能少于六位")
            return
        # 执行登录逻辑
        if userService.login(account, password):
            self.login_result = True
            self.on_closing()
        else:
            # self.config(state="normal")
            messagebox.showerror(title="Show Error", message="账号密码错误")

    def get_current_os(self):
        platform = sys.platform
        if platform.startswith('win'):
            return "Windows"
        elif platform.startswith('linux'):
            return "Linux"
        elif platform == 'darwin':
            return "Mac"
        else:
            return "Unknown"

    def create_login_view(self):
        root = self.root
        root.title("引航计划")
        root.resizable(width=False, height=False)
        # root.overrideredirect(True)
        root.geometry("{0}x{1}+0+0".format(root.winfo_screenwidth(), root.winfo_screenheight()))

        self.canvas = tk.Canvas(root)
        self.canvas.pack(fill="both", expand=True)
        system_type = self.get_current_os()
        if system_type == 'Windows':
            root.wm_attributes("-toolwindow", True)
        elif system_type == 'Linux':
            root.protocol("WM_ICONIFY", self.null_def)
            root.bind("<Unmap>", self.null_def)
        login_bg = os.path.join(os.path.dirname(__file__), "res", "login-poster.png")
        self.bg_img = Image.open(login_bg)
        self.bg_img = self.bg_img.resize((root.winfo_screenwidth(), root.winfo_screenheight()), Image.LANCZOS)
        self.bg_img = ImageTk.PhotoImage(self.bg_img)

        # Create a background image item on the canvas and set its anchor to 'nw'
        self.canvas.create_image(0, 0, image=self.bg_img, anchor="nw", tags="bg")
        self.canvas.config(highlightthickness=0)

        # 获取 Canvas 的宽度和高度
        canvas_width = root.winfo_screenwidth()
        canvas_height = root.winfo_screenheight()

        img_dir = os.path.join(os.path.dirname(__file__), "res", "login_logo.png")
        original_image = Image.open(img_dir)
        resized_image = original_image.resize((320, 130), Image.LANCZOS)  # Resize to 100x100
        self.login_image = ImageTk.PhotoImage(resized_image)
        # 获取图像的宽度和高度
        image_width = self.login_image.width()
        image_height = self.login_image.height()

        # 计算图像在 Canvas 上居中的位置
        x = (canvas_width - image_width) / 2
        y = (canvas_height - image_height) / 2
        self.canvas.create_image(x, y - 150, image=self.login_image, anchor="nw")
        # 添加"账号："标签和输入框
        self.canvas.create_text(x + 40, y + 15, text="账号：".encode("utf-8").decode("utf-8"), anchor="nw")
        self.account_var = tk.StringVar()
        self.account_entry = ttk.Entry(self.canvas, textvariable=self.account_var)
        self.canvas.create_window(x + 190, y + 28, window=self.account_entry)

        # self.account_entry.place(relx=0.46, rely=0.452, relwidth=0.11, relheight=0.025)

        # 添加"密码："标签和输入框
        self.canvas.create_text(x + 40, y + 65, text="密码：".encode("utf-8").decode("utf-8"), anchor="nw")
        self.password_var = tk.StringVar()
        self.password_entry = ttk.Entry(self.canvas, show="*", textvariable=self.password_var)
        self.canvas.create_window(x + 190, y + 75, window=self.password_entry)


        button = ttk.Button(self.canvas, text="登录".encode("utf-8").decode("utf-8"), command=self.login)
        self.account_entry.configure(state="normal")
        self.password_entry.configure(state="normal")
        self.canvas.create_window(x + 160, y + 140, window=button)

        root.mainloop()

    def on_closing(self):
        self.root.destroy()  # 销毁窗口
    def null_def(self):
        pass



import json
import os
import tkinter as tk
from tkinter import filedialog, messagebox, simpledialog, ttk
from tkinter.scrolledtext import ScrolledText

from thonny.utils.SugonUtils import upload_file, get_enable_url, get_tokens, mkdir, search_files, view_files


class FileManagerApp(tk.Tk):
    def __init__(self):
        super().__init__()
        self.title("曙光云文件管理")
        self.geometry("800x600")
        self.button_frame = tk.Frame(self)
        self.button_frame.pack(side=tk.TOP)

        self.back_button = tk.Button(self.button_frame, text="后退", command=self.go_back)
        self.back_button.pack(side=tk.LEFT, anchor=tk.W)

        self.new_button = tk.Button(self.button_frame, text="新建目录", command=self.create_file)
        self.new_button.pack(side=tk.LEFT, padx=5)

        self.upload_button = tk.Button(self.button_frame, text="上传", command=self.upload_file)
        self.upload_button.pack(side=tk.LEFT, padx=5)

        self.upload_button = tk.Button(self.button_frame, text="刷新", command=self.refresh_list)
        self.upload_button.pack(side=tk.LEFT, padx=5)

        self.file_tree = ttk.Treeview(self)
        self.file_tree.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        self.file_info_frame = tk.Frame(self)
        self.file_info_frame.pack(side=tk.RIGHT, fill=tk.BOTH, expand=True)

        self.file_info_label = tk.Label(self.file_info_frame, text="文件信息：")
        self.file_info_label.pack()

        self.open_button = tk.Button(self.file_info_frame, text="打开", command=self.open_file)
        self.open_button.configure(state=tk.DISABLED)
        self.open_button.pack()

        self.folder_close_icon = tk.PhotoImage(file="../res/closed-folder.gif")
        self.data = None
        self.default_text = None
        self.file_name = None
        self.current_path = None
        self.parent_path = None
        self.load_files()

    def load_files(self, folder_path=None):
        print(folder_path)
        self.file_tree.delete(*self.file_tree.get_children())
        token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
        enable_url = get_enable_url(token, "efileUrls")
        search_file = search_files(token, enable_url, 10, 0, None, folder_path)
        self.data = json.loads(search_file)
        self.current_path = self.data["data"]["path"]
        file_list = self.data["data"]["fileList"]
        children_list = self.data["data"]["children"]

        for child in children_list:
            self.file_tree.insert("", tk.END, text=f" {child['label']}", values=child,
                                  image=self.folder_close_icon)
        for file in file_list:
            self.file_tree.insert("", tk.END, text=file["name"], values=file)

        self.file_tree.bind("<<TreeviewSelect>>", self.show_file_info)

    def show_file_info(self, event):
        selected_item = self.file_tree.focus()
        print(selected_item)
        file_info_text = ""
        file = self.file_tree.item(selected_item)
        print(file)
        if file['image']:
            if file['image'][0].startswith("pyimage1"):
                path = file['values'][0].split("'path': ")[1].split(",")[0].strip("'")  # 使用字符串分割提取路径,去除路径中的引号
                file_info_text += f"文件夹名: {file['text']}\n"
                file_info_text += f"文件路径: {path}\n"
                self.open_button.configure(state=tk.NORMAL)
        else:
            file_info_text += f"文件夹名: {file['text']}\n"
            path = file['values'][0].split("'path': ")[1].split(",")[0].strip("'")  # 使用字符串分割提取路径,去除路径中的引号
            file_info_text += f"文件路径: {path}\n"
            owner = file['values'][0].split("'owner': ")[1].split(",")[0].strip("'")
            file_info_text += f"创建者: {owner}\n"
            size = file['values'][0].split("'size': ")[1].split(",")[0].strip("'")
            file_info_text += f"文件大小: {size} bytes\n"
            lastModifiedTime = file['values'][0].split("'lastModifiedTime': ")[1].split(",")[0].strip("'")
            file_info_text += f"最后修改时间: {lastModifiedTime}"
            print(file['values'])
            self.open_button.configure(state=tk.NORMAL)
            # self.open_button.configure(state=tk.DISABLED)

        self.file_info_label.configure(text=file_info_text)

    def open_file(self):
        selected_item = self.file_tree.focus()
        file = self.file_tree.item(selected_item)
        path = file['values'][0].split("'path': ")[1].split(",")[0].strip("'")
        if file['image']:
            print(f"打开文件夹路径: {path}")
            self.parent_path = self.current_path
            self.load_files(folder_path=path)
        else:
            print(f"打开文件路径: {path}")
            token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
            enable_url = get_enable_url(token, "hpcUrls")
            file_content = view_files(token, enable_url, path)
            file_content_o = json.loads(file_content)
            print(file_content_o)
            if file_content_o['code'] == "0":
                if file_content_o['data']['success'] == "true":
                    self.file_name = file['text']
                    self.default_text = file_content_o['data']['data']
                    print(f"文本内容: {self.default_text}")
                    editor_window = tk.Toplevel(self)
                    text_area = ScrolledText(editor_window, wrap=tk.WORD)
                    text_area.insert("1.0", self.default_text)
                    text_area.pack(fill=tk.BOTH, expand=True)

                    def save_on_ctrl_s():
                        text_content = text_area.get("1.0", tk.END)
                        print(text_content)
                        try:
                            with open(self.file_name, "w") as f:
                                f.write(text_content)
                            efile_url = get_enable_url(token, "efileUrls")
                            print(os.path.dirname(path))
                            upload_file(token, efile_url, os.path.dirname(path), "cover", self.file_name)
                            self.default_text = text_content
                            tk.messagebox.showinfo("保存成功", "文本保存成功。")
                        except Exception as e:
                            tk.messagebox.showerror("保存失败", f"保存文本时出错: {str(e)}")

                        # 使用完毕后删除临时文件
                        try:
                            os.remove(self.file_name)
                        except Exception as e:
                            print(f"Failed to delete the temporary file: {str(e)}")

                    editor_window.bind('<Control-s>', save_on_ctrl_s)

                    save_button = tk.Button(editor_window, text="保存", command=save_on_ctrl_s)
                    save_button.pack()

                    def on_close():
                        current_text = text_area.get("1.0", tk.END).strip()
                        if current_text != self.default_text:
                            result = messagebox.askyesnocancel("关闭文本", "是否要在关闭前保存更改？")
                            if result is True:
                                save_on_ctrl_s()
                                editor_window.destroy()
                            elif result is False:
                                editor_window.destroy()
                        else:
                            editor_window.destroy()

                    editor_window.protocol("WM_DELETE_WINDOW", on_close)
                else:
                    # 显示提示信息
                    self.open_button.configure(state=tk.DISABLED)
                    messagebox.showinfo("提示", f"目前仅支持文本文件打开查看")

            else:
                # 显示提示信息
                messagebox.showinfo("提示", f"远程连接失败")

    def go_back(self):
        # 执行返回操作
        # 调用接口或编写逻辑以实现后退功能
        print("Go Back")
        self.load_files(folder_path=self.parent_path)

    def create_file(self):
        # 执行新建文件操作
        # 调用接口或编写逻辑以实现新建文件功能
        print("Create File")
        file_name = simpledialog.askstring("新建目录", "请输入文件名")
        if file_name:
            # 调用接口或编写逻辑以实现创建文件功能
            print(f"Creating File: {file_name}")
            print(f"current_path: {self.current_path}")
            token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
            enable_url = get_enable_url(token, "efileUrls")
            mkdir(token, enable_url, self.current_path + "/" + file_name, "false")
            # 显示提示信息
            messagebox.showinfo("提示", f"创建文件 {file_name} 完成")

    def refresh_list(self):
        # 执行刷新文件列表操作
        # 调用接口或编写逻辑以实现新建文件功能
        print("Refresh List")
        self.load_files(folder_path=self.current_path)

    def upload_file(self):
        # 执行上传文件操作
        # 调用接口或编写逻辑以实现上传文件功能
        print("Upload File")
        file_path = filedialog.askopenfilename()
        if file_path:
            # 调用接口或编写逻辑以实现上传文件功能
            print(f"Uploading File: {file_path}")
            token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
            enable_url = get_enable_url(token, "efileUrls")
            upload_file(token, enable_url, self.current_path, "uncover", file_path)


app = FileManagerApp()
app.mainloop()



import socket

"""
网络连接工具类
"""


class InternetUtil:
    def __init__(self):
        self.network_connections = False

    """
    检查网络连接状况
    """

    def check_internet_connection(self, host="180.76.235.69", port=3306, timeout=5):
        try:
            socket.setdefaulttimeout(timeout)
            socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect((host, port))
            self.network_connections = True
            return self.network_connections
        except OSError:
            self.network_connections = False
            return self.network_connections



import json
import threading
import tkinter as tk
from datetime import datetime, timedelta
from tkinter import messagebox, ttk

from thonny.utils.SugonUtils import (
    get_enable_url,
    get_job_cluster,
    get_job_queues,
    get_tokens,
    search_history_job_detail,
    submit_job,
    view_current_job_list,
    view_history_job_list,
)

from thonny import get_workbench
from thonny.time_select import DateTimePicker
from thonny.ui_utils import (
    CommonDialog,
)

import time

class WorkDialog(CommonDialog):
    def __init__(self, master):
        super().__init__(master=master)
        self.title("曙光作业")
        screen_width = self.winfo_screenwidth()
        self.center_window(screen_width - 100, 600)
        self.token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
        self.enable_url = get_enable_url(self.token, "hpcUrls")
        self.job_cluster = get_job_cluster(self.token, self.enable_url)

        # 创建选项卡控件
        self.tabControl = ttk.Notebook(self)
        self.tabControl.pack(fill="both", expand=1)

        # 创建选项卡1
        self.tab1 = ttk.Frame(self.tabControl)
        self.tabControl.add(self.tab1, text="当前作业")

        # 在选项卡1中添加表格
        columns = ["作业id", "作业名", "应用", "队列", "区域", "开始时间", "运行时长", "状态"]
        self.treeview1 = ttk.Treeview(self.tab1, columns=columns, show="headings")
        for col in columns:
            self.treeview1.heading(col, text=col)
            self.treeview1.column(col, anchor="center")  # 设置列内容居中对齐
        self.treeview1.pack(padx=20, pady=20)

        # 创建按钮
        add_button = tk.Button(self.tab1, text="新增", command=self.add_data)
        add_button.pack(side=tk.LEFT, padx=10)
        delete_button = tk.Button(self.tab1, text="删除", command=self.delete_data)
        delete_button.pack(side=tk.LEFT, padx=10)

        # 创建选项卡2
        self.tab2 = ttk.Frame(self.tabControl)
        self.tabControl.add(self.tab2, text="历史作业")

        delete_button1 = tk.Button(self.tab2, text="查看详情", command=self.look_data)
        delete_button1.pack(side=tk.BOTTOM, pady=50)

        self.headerDiv = tk.Frame(self.tab2)
        self.headerDiv.pack(side=tk.TOP, pady=20, padx=10, anchor="w")

        self.statString = tk.StringVar()
        self.endString = tk.StringVar()
        self.statString.set(self.get_previous_year(30))
        self.endString.set(self.get_previous_year(0))

        start_btn = tk.Button(self.headerDiv, text="选择开始时间", command=self.select_start)
        start_btn.grid(row=0, column=1, padx=30)

        start_lab = tk.Label(self.headerDiv, textvariable=self.statString)
        start_lab.grid(row=1, column=1)

        start_btn = tk.Button(self.headerDiv, text="选择结束时间", command=self.select_end)
        start_btn.grid(row=0, column=2, padx=30)

        end_lab = tk.Label(self.headerDiv, textvariable=self.endString)
        end_lab.grid(row=1, column=2)

        select_btn = tk.Button(self.headerDiv, text="查询", command=self.select_click)
        select_btn.grid(row=0, column=4, padx=30)

        # 在选项卡2中添加空表格

        columnsHis = ["作业id", "作业名", "应用", "队列", "区域", "结束时间", "运行时长", "状态"]
        self.treeview2 = ttk.Treeview(self.tab2, columns=columnsHis, show="headings")
        for col in columnsHis:
            self.treeview2.heading(col, text=col)
            self.treeview2.column(col, anchor="center")  # 设置列内容居中对齐
        self.treeview2.pack(padx=80, pady=80)

        # 绑定选项卡切换事件
        self.tabControl.bind("<<NotebookTabChanged>>", self.on_tab_changed)

        # 初始化数据
        self.data1 = []
        self.data2 = []
        current_job_list = view_current_job_list(
            token=self.token, hpc_prefix=self.enable_url, job_cluster=self.job_cluster
        )
        reqResult = json.loads(current_job_list)["data"]["list"]
        if reqResult is None:
            reqResult = []
        self.soureData2 = reqResult
        for item in reqResult:
            temp = []
            temp.append(item["jobId"])
            temp.append(item["jobName"])
            temp.append(item["appType"])
            temp.append(item["queue"])
            temp.append(item["jobmanagerName"])
            temp.append(item["jobStartTime"])
            temp.append(item["jobRunTime"])
            temp.append(self.checkStatus(item["jobStatus"]))
            self.data1.append(temp)
        current_time = datetime.now()
        formatted_time = current_time.strftime("%Y-%m-%d %H:%M:%S")
        current_time = datetime.now()
        ten_days_later = current_time + timedelta(days=10)
        next_time = ten_days_later.strftime("%Y-%m-%d %H:%M:%S")

        history_job_list = view_history_job_list(
            token=self.token,
            hpc_prefix=self.enable_url,
            job_cluster=self.job_cluster,
            start_time=formatted_time,
            end_time=next_time,
        )
        reqResult = json.loads(history_job_list)["data"]["list"]
        if reqResult is None:
            reqResult = []
        self.soureData = reqResult
        for item in reqResult:
            temp = []
            temp.append(item["jobmanagerId"])
            temp.append(item["jobName"])
            temp.append(item["appType"])
            temp.append(item["queue"])
            temp.append("嘿嘿区域")
            temp.append(item["jobEndTime"])
            temp.append(item["jobWalltimeUsed"])
            temp.append(self.checkStatus(item["jobState"]))
            self.data2.append(temp)

        self.load_data(self.treeview1, self.data1)  # 初始化选项卡1的数据

    def select_start(self):
        self.float_div = tk.Toplevel(self.headerDiv)
        # 设置为模态
        self.float_div.grab_set()
        # 设置宽高
        self.float_div.geometry("600x600")
        self.float_div.title("开始时间:")
        res = DateTimePicker(self.float_div, "开始时间")
        # 添加获取选定时间的操作
        get_time_button = ttk.Button(
            self.float_div, text="确定", command=lambda: self.get_selected_time(res)
        )
        get_time_button.pack(pady=10)

    def select_end(self):
        self.float_div = tk.Toplevel(self.headerDiv)
        # 设置为模态
        self.float_div.grab_set()
        # 设置宽高
        self.float_div.geometry("600x600")
        self.float_div.title("结束时间:")
        res = DateTimePicker(self.float_div, "结束时间")
        # 添加获取选定时间的操作
        get_time_button = ttk.Button(
            self.float_div, text="确定", command=lambda: self.get_selected_time_end(res)
        )
        get_time_button.pack(pady=10)

    def get_selected_time(self, datetime_picker):
        selected_time = datetime_picker.get_selected_datetime()
        # 在这里进行您想要的操作，使用选定的时间 'selected_time'
        self.statString.set(selected_time)
        self.float_div.destroy()

    def get_selected_time_end(self, datetime_picker):
        selected_time = datetime_picker.get_selected_datetime()
        # 在这里进行您想要的操作，使用选定的时间 'selected_time'
        self.endString.set(selected_time)
        self.float_div.destroy()

    def get_previous_year(self, day):
        # 获取当前时间
        current_time = datetime.now()

        # 计算并返回当前时间减一年后的结果
        previous_year = current_time - timedelta(days=day)
        previous_year_str = previous_year.strftime('%Y-%m-%d %H:%M:%S')
        return previous_year_str

    def select_click(self):
        start = self.statString.get()
        end = self.endString.get()

        def request_api():
            nonlocal history_job_list  # 使用nonlocal关键字声明history_job_list为外部变量，以便在函数内部对其进行赋值
            history_job_list = view_history_job_list(
                token=self.token,
                hpc_prefix=self.enable_url,
                job_cluster=self.job_cluster,
                start_time=start,
                end_time=end,
            )
            # 创建一个新线程执行接口请求

        t = threading.Thread(target=request_api)
        t.start()

        t.join(timeout=30)
        if t.is_alive():
            messagebox.showerror('错误', '曙光服务器繁忙，请在空闲时间重试')
            return
        # 获取接口请求开始时间
        history_job_list = view_history_job_list(
            token=self.token,
            hpc_prefix=self.enable_url,
            job_cluster=self.job_cluster,
            start_time=start,
            end_time=end,
        )# 获取接口请求结束时间


        reqResult = json.loads(history_job_list)["data"]["list"]
        if reqResult is None:
            reqResult = []
        self.soureData = reqResult
        for item in reqResult:
            temp = []
            temp.append(item["jobmanagerId"])
            temp.append(item["jobName"])
            temp.append(item["appType"])
            temp.append(item["queue"])
            temp.append("嘿嘿区域")
            temp.append(item["jobEndTime"])
            temp.append(item["jobWalltimeUsed"])
            temp.append(self.checkStatus(item["jobState"]))
            self.data2.append(temp)
        self.load_data(self.treeview2, self.data2)

    def validate_datetime_format(self, input_datetime):
        try:
            datetime.strptime(input_datetime, "%Y-%m-%d %H:%M:%S")
            return True
        except ValueError:
            return False

    def checkStatus(self, type):
        if type == "statR":
            return "运行"
        elif type == "statQ":
            return "排队"
        elif type == "statH":
            return "保留"
        elif type == "statS":
            return "挂起"
        elif type == "statE":
            return "退出"
        elif type == "statC":
            return "完成"
        elif type == "statW":
            return "等待"
        elif type == "statX":
            return "完成"

    def add_data(self):
        # 创建下拉选择框
        self.new_window = tk.Toplevel(master=self)
        self.new_window.grab_set()
        self.new_window.title("新窗口")
        label_submission = tk.Label(self.new_window, text="提交方式:")
        label_submission.pack()
        self.combo_submission = ttk.Combobox(self.new_window, values=["命令行方式"])
        self.combo_submission.pack()
        default_value = "命令行方式"
        self.combo_submission.current(self.combo_submission["values"].index(default_value))

        # 创建标签和输入框
        label_name = tk.Label(self.new_window, text="作业名称:")
        label_name.pack()
        self.entry_name = tk.Entry(self.new_window)
        self.entry_name.pack()

        # 创建选择文件目录按钮
        label_directory = tk.Label(self.new_window, text="工作目录:")
        label_directory.pack()
        self.entry_directory = tk.Entry(self.new_window)
        self.entry_directory.pack()
        # button_select_directory = tk.Button(self, text="选择文件目录", command=self.select_directory)
        # button_select_directory.pack()

        # 创建命令行方式的文本输入框
        label_command = tk.Label(self.new_window, text="命令行方式:")
        label_command.pack()
        self.text_command = tk.Text(self.new_window, height=10)
        self.text_command.pack()
        default_text = "sleep 5"
        self.text_command.insert("1.0", default_text)

        # 创建提交按钮
        button_submit = tk.Button(self.new_window, text="提交", command=self.submit_homework)
        button_submit.pack()

    def delete_data(self):
        # 获取选中行的索引
        selection = self.treeview1.selection()
        if selection:
            index = int(self.treeview1.index(selection[0]))
            # messagebox.showinfo('选择的下标', '你选择的下标是' + str(index))
            res_snow = self.soureData2[index]
            print("获取的结果", res_snow)
        else:
            messagebox.showerror("错误", "你啥也没选")

    def look_data(self):
        # 获取选中行的索引
        selection = self.treeview2.selection()
        if selection:
            index = int(self.treeview2.index(selection[0]))
            row = self.soureData[index]
            job_detail = search_history_job_detail(
                token=self.token,
                hpc_prefix=self.enable_url,
                job_id=str(row["jobId"]),
                jobmanager_id=str(row["jobmanagerId"]),
                acct_time=str(row["jobEndTime"]),
            )  # 不传入账时间（结束时间），查询巨慢
            resJson = json.loads(job_detail)
            print(666, resJson)
            data_dict = resJson["data"]
            self.new_window1 = tk.Toplevel(master=self)
            self.new_window1.grab_set()
            self.new_window1.title("新窗口")
            self.new_window1.geometry("500x500")
            # 创建滚动区域
            scrollbar = ttk.Scrollbar(self.new_window1)
            scrollbar.pack(side=tk.RIGHT, fill=tk.Y)
            # 创建滚动文本框，并设置为不可编辑状态
            text_box = tk.Text(self.new_window1, yscrollcommand=scrollbar.set)

            text_box.pack(fill=tk.BOTH, expand=True)
            # 配置滚动条和滚动文本框的关联
            scrollbar.config(command=text_box.yview)
            # 将数据字典内容添加到滚动文本框中
            for key, value in data_dict.items():
                text_box.insert(tk.END, f"{self.checkStatusKey(key)}: {value}\n")
        else:
            messagebox.showerror("错误", "你啥也没选")

    def checkStatusKey(self, type):
        status_dict = {
            "acctTime": "记账时间",
            "appType": "应用类型",
            "command": "作业脚本的位置",
            "commandExist": "作业脚本是否存在",
            "cpuNuclearHour": "CPU核时",
            "cpuNuclearSec": "CPU核秒",
            "cpuUnitPrice": "作业提交时当时的CPU单价",
            "dcuCardHour": "DCU卡时",
            "dcuCardSec": "DCU卡秒",
            "dcuUnitPrice": "作业提交时当时的DCU单价",
            "efficiencyCpu": "CPU效率",
            "exclusiveCputime": "作业独占的cputime，单位为秒，当作业为独占作业时有值，否则为0",
            "exclusiveMem": "作业独占的mem",
            "exclusiveWalltime": "作业独占的walltime，单位为秒，当作业为独占作业时有值，否则为0",
            "goldenable": "作业提交时当时的计费状态",
            "gpuCardHour": "GPU卡时",
            "gpuCardSec": "GPU卡秒",
            "gpuUnitPrice": "作业提交时当时的GPU单价",
            "groupName": "用户组名",
            "historyAccount": "作业运行结束后当时作业提交者所属的账号",
            "historyQueuerate": "作业提交时当时的队列费率",
            "isSinglejob": "是否为独占节点的作业",
            "jobCpuTime": "作业占用的CPU时间",
            "jobDcuNum": "作业使用的DCU数",
            "jobEndTime": "作业结束时间",
            "jobExecGpus": "作业占用的Gpu节点",
            "jobExecHost": "作业执行节点",
            "jobExitStatus": "作业退出代码",
            "jobGpuNum": "作业使用的GPU核数",
            "jobId": "作业ID",
            "jobMemUsed": "作业使用的物理内存数，单位kb",
            "jobName": "作业名",
            "jobProcNum": "作业使用的处理器数(表示核数)",
            "jobQueueTime": "作业入队时间，对应记账属性qtime",
            "jobReqCpu": "申请CPU",
            "jobReqDcu": "申请DCU",
            "jobReqGpu": "申请GPU",
            "jobReqMem": "申请内存",
            "jobReqNodes": "申请节点数",
            "jobResponseTime": "作业响应时间JobEndTime-JobQueueTime,单位为秒",
            "jobStartTime": "作业启动时间",
            "jobState": "作业状态",
            "jobVmemUsed": "作业使用的虚拟内存，单位kb",
            "jobWaitTime": "作业等待时间JobStartTime-JobQueueTime,单位为秒",
            "jobWalltimeUsed": "作业实际使用的Walltime,单位为秒",
            "jobmanagerId": "区域ID",
            "jobmanagerName": "区域名称",
            "needNodes": "分配的节点名或节点数",
            "nodect": "分配的节点数",
            "owner": "作业拥有者",
            "queue": "队列",
            "scale": "作业的规模",
            "shareCputime": "作业共享的cputime，单位为秒，当作业为非独占作业时有值，否则为0",
            "shareMem": "作业共享的mem，单位kb，当作业为非独占作业时有值，否则为0",
            "shareWalltime": "作业共享的walltime，单位为秒，当作业为非独占作业时有值，否则为0",
            "isSinglejob": "是否为独立结点作业",
            "startCount": "作业的启动次数，在对应的数据库表中没有这个字段",
            "userName": "用户名",
            "walltime": "提交作业时请求的Walltime或系统默认的Walltime,单位为秒",
            "workdir": "作业的工作路径",
        }

        if type in status_dict:
            if type == "jobState":
                return self.checkStatus(status_dict[type])
            return status_dict[type]
        else:
            return "未知状态"

    def submit_homework(self):
        # 获取下拉选择框的值
        submission_method = self.combo_submission.get()

        # 获取文本输入框的值
        job_name = self.entry_name.get()
        working_directory = self.entry_directory.get()
        command_line = self.text_command.get("1.0", tk.END)

        if submission_method == "命令行方式":
            print("命令行:", command_line)
            if not command_line.strip():
                # command_line为空
                messagebox.showerror(
                    "提交作业",
                    "命令行脚本不能为空\n\n",
                    master=get_workbench(),
                )
            else:
                if command_line:
                    # 在这里可以将作业提交到服务器或进行其他处理
                    # todo 曙光账号来源
                    token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
                    enable_url = get_enable_url(token, "hpcUrls")
                    job_cluster = get_job_cluster(token, enable_url)
                    job_queues = get_job_queues(token, enable_url, "acsx6nwsm6", job_cluster)
                    job_info = submit_job(
                        token,
                        enable_url,
                        job_cluster,
                        job_queues,
                        command_line,
                        job_name,
                        working_directory,
                    )
                    print("提交请求:", job_info)
                    # 提交成功提示窗口
                    messagebox.showinfo("提交成功", "作业已成功提交！")
                    # 关闭弹出框
                    self.new_window.destroy()

                    current_job_list = view_current_job_list(
                        token=self.token, hpc_prefix=self.enable_url, job_cluster=self.job_cluster
                    )
                    reqResult = json.loads(current_job_list)["data"]["list"]
                    if reqResult is None:
                        reqResult = []
                    reqResult
                    for item in reqResult:
                        temp = []
                        temp.append(item["jobId"])
                        temp.append(item["jobName"])
                        temp.append(item["appType"])
                        temp.append(item["queue"])
                        temp.append(item["jobmanagerName"])
                        temp.append(item["jobStartTime"])
                        temp.append(item["jobRunTime"])
                        temp.append(self.checkStatus(item["jobStatus"]))
                        self.data1.append(temp)
                    self.load_data(self.treeview1, self.data1)

    def load_data(self, treeview, data):
        # 清除表格数据
        treeview.delete(*treeview.get_children())

        # 添加新数据
        for row in data:
            treeview.insert("", tk.END, values=row)

    def on_tab_changed(self, event):
        current_tab = event.widget.tab(event.widget.select(), "text")
        if current_tab == "当前作业":
            self.load_data(self.treeview1, self.data1)
        elif current_tab == "历史作业":
            self.load_data(self.treeview2, self.data2)

    def center_window(self, width, height):
        screen_width = self.winfo_screenwidth()
        screen_height = self.winfo_screenheight()

        x = (screen_width - width) // 2
        y = (screen_height - height) // 2

        self.geometry(f"{width}x{height}+{x}+{y}")



from thonny.utils.SugonUtils import get_tokens, get_enable_url, mkdir

# 1、获取令牌的方法
token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
print(token)
# 2、获取对应操作授权地址 efileUrls hpcUrls eshellUrls
enable_url = get_enable_url(token, "efileUrls")
print(enable_url)
# 文件列表，文件搜索
# search_file = search_files(token, enable_url, 10, 0, None, None)
# print(search_file)
# # 3、作业集群获取
# job_cluster = get_job_cluster(token, enable_url)
# print(job_cluster)
# # 4、作业队列获取
# job_queues = get_job_queues(token, enable_url, "acsx6nwsm6", job_cluster)
# print(job_queues)
# # 5、提交作业
# submit_job = submit_job(token, enable_url, job_cluster, job_queues, "ls", "007", "")
# print(submit_job)
# 6、查看文件
# file_content = view_files(token, enable_url, "/public/home/acsx6nwsm6/test.py")
# print(file_content)
# 7、文件创建
mkdir_info = mkdir(token, enable_url, "/public/home/acsx6nwsm6/test20230821.txt", "false")
print(mkdir_info)
# 8、文件上传
# upload_info = upload_file(token, enable_url, "/public/home/acsx6nwsm6/test20230821/", "uncover", "D:/AFoooooo/ai1.jpg")
# print(upload_info)
# 9、判断文件/文件夹是否存在
# exist_info = exist_file(token, enable_url, "/public/home/acsx6nwsm6/test20230821/")
# print(exist_info)
# 10、删除文件/文件夹是否存在
# remove_info = remove_file(token, enable_url, "/public/home/acsx6nwsm6/test20230821/ai1.jpg", "false")
# print(remove_info)
# 11、查询当前作业列表
# current_job_list = view_current_job_list(token=token, hpc_prefix=enable_url, job_cluster=job_cluster)
# print(current_job_list)
# 12、查询历史作业列表
# history_job_list = view_history_job_list(token=token, hpc_prefix=enable_url, job_cluster=job_cluster,
#                                          start_time="2023-08-15 00:00:00",
#                                          end_time="2023-08-21 23:59:59")
# print(history_job_list)
# 13、删除作业
# delete_job_info = delete_job(token, enable_url, "1573088268,acsx6nwsm6:40615631:")
# print(delete_job_info)
# 14、查询作业详细信息(只能查当前作业列表)
# current_job_detail = search_current_job_detail(token, enable_url, "40618124")
# print(current_job_detail)
# 15、查询历史作业详细信息(只能查历史作业列表)
# job_detail = search_history_job_detail(token=token, hpc_prefix=enable_url, job_id="40618124",
#                                        jobmanager_id="1573088268")#  不传入账时间（结束时间），查询巨慢
# job_detail = search_history_job_detail(token=token, hpc_prefix=enable_url, job_id="40618124",
#                                        jobmanager_id="1573088268", acct_time="2023-08-21 11:50:34")
# history_job_detail = search_history_job_detail(token, enable_url, "40618124", "1573088268", "2023-08-21 11:50:34")
# history_job_detail = search_history_job_detail(token, enable_url, "40615631", "1573088268", "2023-08-21 10:24:12")
# print(history_job_detail)




# from PIL import Image
#
# # 打开原始图片
# image = Image.open('../thonny/res/ac.png')
#
# # 将图片调整为16x16大小
# resized_image = image.resize((16, 16), Image.ANTIALIAS)
#
# # 保存调整后的图片
# resized_image.save('../thonny/res/ac_16x16.png')



import tkinter as tk
from tkinter import messagebox, ttk

from thonny.sqlite3_sql import QuestionService


class ExamDetailsPage:
    def __init__(self, root, questions, exam_id, type, time):
        self.root = root
        self.questions = questions
        self.exam_id = exam_id
        self.type = type
        self.time = time

        # 创建题目数组
        # self.questions = [
        #     {"id": 1, "type": "SINGLE_CHOICE", "title": "问题1", "options": ["选项1", "选项2", "选项3", "选项4"]},
        #     {"id": 2, "type": "MULTIPLE_CHOICE", "title": "问题2", "options": ["选项1", "选项2", "选项3", "选项4"]},
        #     {"id": 3, "type": "JUDGE", "title": "问题3", "options": ["正确", "错误"]},
        #     {"id": 5, "type": "SINGLE_CHOICE", "title": "问题5", "options": ["选项1", "选项2", "选项3", "选项4"]},
        #     {"id": 6, "type": "MULTIPLE_CHOICE", "title": "问题6", "options": ["选项1", "选项2", "选项3", "选项4"]}
        # ]

        # 创建控件
        self.exam_title_label = tk.Label(self.root, text="试卷标题")  # 试卷标题标签
        self.exam_title_label.pack()

        self.total_score_label = tk.Label(self.root, text="总分：100")  # 总分标签
        self.total_score_label.pack()

        self.passing_score_label = tk.Label(self.root, text="及格分数：60")  # 及格分数标签
        self.passing_score_label.pack()

        self.exam_time_label = tk.Label(
            self.root, text="考试时间：" + str(self.time / 60) + "分钟"
        )  # 考试时间标签
        self.exam_time_label.pack()

        self.total_seconds = self.time
        # self.total_seconds = 5
        self.hours = 0  # 时
        self.minutes = 0  # 分
        self.seconds = 0  # 秒

        self.countdown_label = tk.Label(self.root, text="考试结束倒计时: ")
        self.countdown_label.pack()
        self.run()

        # 创建滚动条
        self.scrollbar = ttk.Scrollbar(self.root)
        self.scrollbar.pack(side=tk.RIGHT, fill=tk.Y)

        # 创建 Canvas
        self.canvas = tk.Canvas(self.root, yscrollcommand=self.scrollbar.set)
        self.canvas.pack(fill=tk.BOTH, expand=1)

        self.scrollbar.config(command=self.canvas.yview)

        self.question_frame = tk.Frame(self.canvas, highlightthickness=0)

        self.canvas.create_window((0, 0), window=self.question_frame, anchor=tk.NW)

        self.questions_labels = []  # 题目标签列表
        self.radio_vars = []  # SINGLE_CHOICE题选项变量列表
        self.determine = []  # JUDGE题选项变量列表
        self.check_vars = []  # MULTIPLE_CHOICE题选项变量列表

        # self.res

        for question in self.questions:
            question_label = tk.Label(
                self.question_frame,
                text=f"{self.top_ic_type(question['type'])}题目：{question['title']},分值：{question['fraction']}, 难度：{str(question['difficulty']) + '颗星'}",
            )
            question_label.pack(anchor="w", fill="x")
            self.questions_labels.append(question_label)

            if question["type"] == "SINGLE_CHOICE" or question["type"] == "MULTIPLE_CHOICE":
                label_frame = tk.Frame(self.question_frame)  # 标签框架
                label_frame.pack(anchor="w", fill="x", padx=10)

                options_frame = tk.Frame(label_frame)  # 选项框架
                options_frame.pack(side=tk.LEFT)

                if question["type"] == "SINGLE_CHOICE":
                    radio_var = tk.IntVar()  # SINGLE_CHOICE题选项变量
                    self.radio_vars.append(radio_var)

                    for index, option in enumerate(question["options"]):
                        option_radio = tk.Radiobutton(
                            options_frame, text=option[1], variable=radio_var, value=index + 1
                        )
                        option_radio.pack(anchor="w", fill="x", padx=10)
                elif question["type"] == "MULTIPLE_CHOICE":
                    check_vars_list = []  # MULTIPLE_CHOICE题选项变量列表
                    for i, option in enumerate(question["options"]):
                        check_var = tk.IntVar()  # MULTIPLE_CHOICE题选项变量
                        check_vars_list.append(check_var)
                        option_check = tk.Checkbutton(
                            options_frame, text=option[1], variable=check_var
                        )
                        option_check.pack(anchor="w", fill="x", padx=10)
                    self.check_vars.append(check_vars_list)

            elif question["type"] == "JUDGE":
                judge_var = tk.IntVar()  # JUDGE题选项变量

                for item in question["options"]:
                    option_radio1 = tk.Radiobutton(
                        self.question_frame, text=item[1], variable=judge_var, value=item[0]
                    )
                    option_radio1.pack(anchor="w", fill="x", padx=10)

                self.determine.append(judge_var)

        self.submit_button = tk.Button(self.root, text="提交", command=self.submit_exam)  # 提交按钮
        self.submit_button.pack(pady=20)

        self.question_frame.bind("<Configure>", self.on_frame_configure)

    def start_countdown(self):
        if self.total_seconds > 0:
            self.calculate_time()

            self.countdown_label[
                "text"
            ] = f"考试结束倒计时: {self.hours:02d}:{self.minutes:02d}:{self.seconds:02d}"

            self.total_seconds -= 1
            self.root.after(1000, self.start_countdown)
        else:
            messagebox.showinfo("考试结束", "时间到了！")
            self.submit_exam()

    def calculate_time(self):
        self.hours = self.total_seconds // 3600
        self.minutes = (self.total_seconds % 3600) // 60
        self.seconds = self.total_seconds % 60

    def run(self):
        self.start_countdown()

    def top_ic_type(self, type):
        if type == "SINGLE_CHOICE":
            return "单选题: "
        elif type == "MULTIPLE_CHOICE":
            return "多选题: "
        elif type == "JUDGE":
            return "判断题: "
        else:
            return "未知类型题目: "

    def on_frame_configure(self, event):
        self.canvas.configure(scrollregion=self.canvas.bbox("all"))

    def submit_exam(self):
        request_arr = []
        # SINGLE_CHOICE开始下标
        radio_index = 0
        # MULTIPLE_CHOICE开始下标
        checkbox_index = 0
        # JUDGE开始下标
        determine_index = 0
        for index, question in enumerate(self.questions):
            question_id = question["id"]
            question_type = question["type"]
            if question_type == "SINGLE_CHOICE":
                selected_option = question["options"][self.radio_vars[radio_index].get() - 1]
                radio_index += 1
                request_arr.append(
                    {
                        "question_id": "题目id" + str(question_id),
                        "type": "SINGLE_CHOICE",
                        "user_select": selected_option,
                        "source": question,
                    }
                )
            elif question_type == "MULTIPLE_CHOICE":
                temp_dx = 0
                tempArr = []
                for item in self.check_vars[checkbox_index]:
                    if item.get() == 1:
                        tempArr.append(question["options"][temp_dx])
                    temp_dx += 1
                request_arr.append(
                    {
                        "question_id": "题目id" + str(question_id),
                        "type": "MULTIPLE_CHOICE",
                        "user_select": tempArr,
                        "source": question,
                    }
                )
                checkbox_index += 1
            elif question_type == "JUDGE":
                self.determine[determine_index].get()
                request_arr.append(
                    {
                        "question_id": "题目id" + str(question_id),
                        "type": "JUDGE",
                        "user_select": self.determine[determine_index].get(),
                        "source": question,
                    }
                )
                determine_index += 1
        temp_req_arr = []
        for item in request_arr:
            print(item)
            temp = item["source"]
            res = temp["correct"]
            str_list = [str(num) for num in res]
            answer_ids = ",".join(str_list)
            user_select_arr = []
            if type(item["user_select"]) == list:
                for i in item["user_select"]:
                    user_select_arr.append(i[0])
                str_user_list = [str(num) for num in user_select_arr]
                user_select = ",".join(str_user_list)
            elif type(item["user_select"]) == tuple:
                user_select = str(item["user_select"][0])
            else:
                temp
                user_select = str(item["user_select"])
            temp_req_arr.append(
                {
                    "stem_id": temp["id"],
                    "stem_type": temp["type"],
                    "stem_score": temp["fraction"],
                    "answer_ids": user_select,
                    "correct_answer_ids": answer_ids,
                }
            )

        qus = QuestionService()
        if self.type == "开始考试":
            request_obj = {"exam_id": self.exam_id, "stems": temp_req_arr}
            qus.save_exam_answers(request_obj)
            messagebox.showinfo("成功", "提交成功")
        else:
            request_obj = {"homework_id": self.exam_id, "stems": temp_req_arr}
            qus.save_homework_answers(request_obj)
            messagebox.showinfo("成功", "提交成功")
        self.root.destroy()



import tkinter as tk
from tkinter import messagebox

from thonny.exam_details import ExamDetailsPage
from thonny.sqlite3_sql import QuestionService


class ExamListApp:
    def __init__(self, root, exam_list, type):
        self.root = root
        self.exam_list = exam_list
        self.type = type
        self.create_exam_list()

    def create_exam_list(self):
        for i, exam in enumerate(self.exam_list):
            subject_label = tk.Label(self.root, text=exam["id"])
            subject_label.grid(row=i + 1, column=0, padx=5, pady=5)

            date_label = tk.Label(self.root, text=exam["name"])
            date_label.grid(row=i + 1, column=1, padx=10, pady=5)

            time_label = tk.Label(self.root, text=exam["time"])
            time_label.grid(row=i + 1, column=2, padx=5, pady=5)

            time_label = tk.Label(self.root, text=exam["description"])
            time_label.grid(row=i + 1, column=3, padx=10, pady=5)

            start_button = tk.Button(
                self.root, text=self.type, command=lambda exam=exam: self.start_exam(exam)
            )
            start_button.grid(row=i + 1, column=4, padx=10, pady=5)

    def find_correct(self, options):
        tempArr = []
        for item in options:
            if item[3] == 1:
                tempArr.append(item[0])
        return tempArr

    def start_exam(self, exam):
        qus = QuestionService()

        goto_list = []
        detail = qus.question_detail(exam["question_id"])

        # {"id": 1, "type": "SINGLE_CHOICE", "title": "问题1", "options": ["选项1", "选项2", "选项3", "选项4"]},
        for item in detail:
            # print('后端返回', item)
            goto_list.append(
                {
                    "id": item[0],
                    "type": item[2],
                    "title": item[1],
                    "fraction": item[3],
                    "options": item[4],
                    "difficulty": item[5],
                    "correct": self.find_correct(item[4]),
                }
            )

        self.exam_details = tk.Toplevel(self.root)
        ex_w = self.exam_details.winfo_screenwidth()
        ex_h = self.exam_details.winfo_screenheight()
        self.exam_details.grab_set()
        self.exam_details.title("详情")
        # 绑定窗口关闭事件
        self.exam_details.protocol("WM_DELETE_WINDOW", self.on_exam_details_close)
        self.exam_details.geometry(
            "600x600+" + str(int(ex_w / 2) - 300) + "+" + str(int(ex_h / 2) - 300)
        )
        ExamDetailsPage(
            self.exam_details,
            questions=goto_list,
            exam_id=exam["id"],
            type=self.type,
            time=exam["time"],
        )

    def on_exam_details_close(self):
        # 弹出确认对话框
        if messagebox.askokcancel("退出", "确定要关闭吗,关闭题目不保存哦？"):
            # 用户点击确定，关闭窗口
            self.exam_details.destroy()


# if __name__ == '__main__':
#     root = tk.Tk()
#     root.title("考试列表")
#
#     header_labels = ['科目', '日期', '时间']
#     for i, label in enumerate(header_labels):
#         header_label = tk.Label(root, text=label, font=('bold', 12))
#         header_label.grid(row=0, column=i, padx=10, pady=5)
#
#     app = ExamListApp(root)
#
#     root.mainloop()



# -*- coding: utf-8 -*-
import json
import os.path
import re
import sys
import tkinter as tk
from logging import exception, getLogger
from tkinter import filedialog, messagebox, simpledialog, ttk
from tkinter.scrolledtext import ScrolledText
from typing import Optional, Union

from _tkinter import TclError
from thonny.utils.SugonUtils import (
    get_enable_url,
    get_job_cluster,
    get_job_queues,
    get_tokens,
    mkdir,
    search_files,
    submit_job,
    upload_file,
    view_files,
)

from thonny import get_runner, get_workbench, ui_utils
from thonny.base_file_browser import ask_backend_path, choose_node_for_file_operations
from thonny.codeview import BinaryFileException, CodeView, CodeViewText
from thonny.common import (
    REMOTE_PATH_MARKER,
    InlineCommand,
    TextRange,
    ToplevelResponse,
    is_local_path,
    is_remote_path,
    is_same_path,
    normpath_with_actual_case,
    universal_dirname,
)
from thonny.custom_notebook import CustomNotebook, CustomNotebookTab
from thonny.homework_and_exam import homeworkAndExamDialog
from thonny.languages import tr
from thonny.misc_utils import running_on_mac_os, running_on_windows
from thonny.tktextext import rebind_control_a
from thonny.ui_utils import (
    CommonDialog,
    askopenfilename,
    asksaveasfilename,
    get_beam_cursor,
    select_sequence,
)
from thonny.video import VideoDialog
from thonny.work import WorkDialog

PYTHON_FILES_STR = tr("Python files")
_dialog_filetypes = [(PYTHON_FILES_STR, ".py .pyw .pyi"), (tr("all files"), ".*")]

PYTHON_EXTENSIONS = {"py", "pyw", "pyi"}
PYTHONLIKE_EXTENSIONS = set()

logger = getLogger(__name__)


class EditorCodeViewText(CodeViewText):
    """Allows separate class binding for CodeViewTexts which are inside editors"""

    def __init__(self, master=None, cnf={}, **kw):
        super().__init__(
            master=master,
            cnf=cnf,
            **kw,
        )
        self.bindtags(self.bindtags() + ("EditorCodeViewText",))


class Editor(ttk.Frame):
    # self 类本身  master为调用方法的类
    def __init__(self, master):
        # ttk.Frame小部件（widget）基于 Frame 类的一个变体，但使用了 Themed Tkinter（ttk）模块的样式。ttk.Frame 提供了更现代和主题化的外观，与传统的 tkinter
        # 控件相比，它具有更多的样式和定制选项
        ttk.Frame.__init__(self, master)
        assert isinstance(master, EditorNotebook)
        # 调用方法赋值
        self.notebook = master  # type: EditorNotebook

        # parent of codeview will be workbench so that it can be maximized
        # 自定义编辑器
        self._code_view = CodeView(
            get_workbench(),
            propose_remove_line_numbers=True,
            font="EditorFont",
            text_class=EditorCodeViewText,
            cursor=get_beam_cursor(),
        )
        get_workbench().event_generate(
            "EditorTextCreated", editor=self, text_widget=self.get_text_widget()
        )

        self._code_view.grid(row=0, column=0, sticky=tk.NSEW, in_=self)
        self._code_view.home_widget = self  # don't forget home
        self.maximizable_widget = self._code_view

        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=1)

        self._filename = None
        self._last_known_mtime = None

        self._code_view.text.bind("<<Modified>>", self._on_text_modified, True)
        self._code_view.text.bind("<<TextChange>>", self._on_text_change, True)
        self._code_view.text.bind("<Control-Tab>", self._control_tab, True)

        get_workbench().bind("DebuggerResponse", self._listen_debugger_progress, True)
        get_workbench().bind("ToplevelResponse", self._listen_for_toplevel_response, True)

        self.update_appearance()

    def get_text_widget(self):
        return self._code_view.text

    def get_code_view(self):
        # TODO: try to get rid of this
        return self._code_view

    def get_content(self) -> str:
        return self._code_view.get_content()

    def set_filename(self, path):
        self._filename = path

    def get_filename(self, try_hard=False):
        if self._filename is None and try_hard:
            self.save_file()

        return self._filename

    def get_identifier(self):
        if self._filename:
            return self._filename
        else:
            return str(self.winfo_id())

    def get_title(self):
        if self.get_filename() is None:
            result = tr("<untitled>")
        elif is_remote_path(self.get_filename()):
            path = extract_target_path(self.get_filename())
            name = path.split("/")[-1]
            result = "[ " + name + " ]"
        else:
            result = os.path.basename(self.get_filename())

        if self.is_modified():
            result += " *"

        return result

    def check_for_external_changes(self):
        if self._filename is None:
            return

        if is_remote_path(self._filename):
            return

        if self._last_known_mtime is None:
            return

        elif not os.path.exists(self._filename):
            self.notebook.select(self)

            if messagebox.askyesno(
                tr("File is gone"),
                tr("Looks like '%s' was deleted or moved.") % self._filename
                + "\n\n"
                + tr("Do you want to also close the editor?"),
                master=self,
            ):
                self.notebook.close_editor(self)
            else:
                self.get_text_widget().edit_modified(True)
                self._last_known_mtime = None

        elif os.path.getmtime(self._filename) != self._last_known_mtime:
            skip_confirmation = not self.is_modified() and get_workbench().get_option(
                "edit.auto_refresh_saved_files"
            )
            if not skip_confirmation:
                self.notebook.select(self)

            if skip_confirmation or messagebox.askyesno(
                tr("External modification"),
                tr("Looks like '%s' was modified outside of the editor.") % self._filename
                + "\n\n"
                + tr(
                    "Do you want to discard current editor content and reload the file from disk?"
                ),
                master=self,
            ):
                prev_location = self.get_text_widget().index("insert")
                self._load_file(self._filename, keep_undo=True)
                try:
                    self.get_text_widget().mark_set("insert", prev_location)
                    self.see_line(int(prev_location.split(".")[0]))
                except Exception:
                    logger.exception("Could not restore previous location")

            self._last_known_mtime = os.path.getmtime(self._filename)

    def get_long_description(self):
        if self._filename is None:
            result = tr("<untitled>")
        else:
            result = self._filename

        try:
            index = self._code_view.text.index("insert")
            if index and "." in index:
                line, col = index.split(".")
                result += "  @  {} : {}".format(line, int(col) + 1)
        except Exception:
            exception("Finding cursor location")

        return result

    def _load_file(self, filename, keep_undo=False):
        try:
            if is_remote_path(filename):
                result = self._load_remote_file(filename)
            else:
                result = self._load_local_file(filename, keep_undo)
                if not result:
                    return False
        except BinaryFileException:
            messagebox.showerror(
                tr("Problem"), tr("%s doesn't look like a text file") % (filename,), master=self
            )
            return False
        except SyntaxError as e:
            assert "encoding" in str(e).lower()
            messagebox.showerror(
                tr("Problem loading file"),
                tr(
                    "This file seems to have problems with encoding.\n\n"
                    "Make sure it is in UTF-8 or contains proper encoding hint."
                ),
                master=self,
            )
            return False

        self.update_appearance()
        return True

    def _load_local_file(self, filename, keep_undo=False):
        with open(filename, "rb") as fp:
            source = fp.read()

        # Make sure Windows filenames have proper format
        filename = normpath_with_actual_case(filename)
        self._filename = filename
        self.update_file_type()
        self._last_known_mtime = os.path.getmtime(self._filename)

        get_workbench().event_generate("Open", editor=self, filename=filename)
        if not self._code_view.set_content_as_bytes(source, keep_undo):
            return False
        self.get_text_widget().edit_modified(False)
        self._code_view.focus_set()
        self.notebook.remember_recent_file(filename)
        return True

    def _load_remote_file(self, filename):
        self._filename = filename
        self.update_file_type()
        self._code_view.set_content("")
        self._code_view.text.set_read_only(True)

        target_filename = extract_target_path(self._filename)

        self.update_title()
        response = get_runner().send_command_and_wait(
            InlineCommand(
                "read_file", path=target_filename, description=tr("Loading %s") % target_filename
            ),
            dialog_title=tr("Loading"),
        )

        if response.get("error"):
            # TODO: make it softer
            raise RuntimeError(response["error"])

        content = response["content_bytes"]
        self._code_view.text.set_read_only(False)
        if not self._code_view.set_content_as_bytes(content):
            return False
        self.get_text_widget().edit_modified(False)
        self.update_title()
        return True

    def is_modified(self):
        return bool(self._code_view.text.edit_modified())

    def save_file_enabled(self):
        return self.is_modified() or not self.get_filename()

    def save_file(self, ask_filename=False, save_copy=False, node=None) -> Optional[str]:
        if self._filename is not None and not ask_filename:
            save_filename = self._filename
            get_workbench().event_generate("Save", editor=self, filename=save_filename)
        else:
            save_filename = self.ask_new_path(node)

            if not save_filename:
                return None

            if self.notebook.get_editor(save_filename) is not None:
                messagebox.showerror(
                    tr("File is open"),
                    tr(
                        "This file is already open in Thonny.\n\n"
                        "If you want to save with this name,\n"
                        "close the existing editor first!"
                    ),
                    master=get_workbench(),
                )
                return None

            get_workbench().event_generate(
                "SaveAs", editor=self, filename=save_filename, save_copy=save_copy
            )

        content_bytes = self._code_view.get_content_as_bytes()

        if is_remote_path(save_filename):
            result = self.write_remote_file(save_filename, content_bytes, save_copy)
        else:
            result = self.write_local_file(save_filename, content_bytes, save_copy)

        if not result:
            return None

        if not save_copy:
            self._filename = save_filename
            self.update_file_type()

        self.update_title()
        return save_filename

    def write_local_file(self, save_filename, content_bytes, save_copy):
        process_shebang = content_bytes.startswith(b"#!/") and get_workbench().get_option(
            "file.make_saved_shebang_scripts_executable"
        )
        if process_shebang:
            content_bytes = content_bytes.replace(b"\r\n", b"\n")

        try:
            f = open(save_filename, mode="wb")
            f.write(content_bytes)
            f.flush()
            # Force writes on disk, see https://learn.adafruit.com/adafruit-circuit-playground-express/creating-and-editing-code#1-use-an-editor-that-writes-out-the-file-completely-when-you-save-it
            os.fsync(f)
            f.close()
            if process_shebang:
                os.chmod(save_filename, 0o755)
            if not save_copy or save_filename == self._filename:
                self._last_known_mtime = os.path.getmtime(save_filename)
            get_workbench().event_generate(
                "LocalFileOperation", path=save_filename, operation="save"
            )
        except PermissionError:
            messagebox.showerror(
                tr("Permission Error"),
                tr("Looks like this file or folder is not writable."),
                master=self,
            )
            return False

        if not save_copy or save_filename == self._filename:
            self.notebook.remember_recent_file(save_filename)

        if not save_copy or save_filename == self._filename:
            self._code_view.text.edit_modified(False)

        return True

    def update_file_type(self):
        if self._filename is None:
            self._code_view.set_file_type(None)
        else:
            ext = self._filename.split(".")[-1].lower()
            if ext in PYTHON_EXTENSIONS:
                file_type = "python"
            elif ext in PYTHONLIKE_EXTENSIONS:
                file_type = "pythonlike"
            else:
                file_type = None

            self._code_view.set_file_type(file_type)

        self.update_appearance()

    def write_remote_file(self, save_filename, content_bytes, save_copy):
        if get_runner().ready_for_remote_file_operations(show_message=True):
            target_filename = extract_target_path(save_filename)

            result = get_runner().send_command_and_wait(
                InlineCommand(
                    "write_file",
                    path=target_filename,
                    content_bytes=content_bytes,
                    editor_id=id(self),
                    blocking=True,
                    description=tr("Saving to %s") % target_filename,
                    make_shebang_scripts_executable=get_workbench().get_option(
                        "file.make_saved_shebang_scripts_executable"
                    ),
                ),
                dialog_title=tr("Saving"),
            )

            if "error" in result:
                messagebox.showerror(tr("Could not save"), str(result["error"]))
                return False

            if not save_copy:
                self._code_view.text.edit_modified(False)

            self.update_title()

            # NB! edit_modified is not falsed yet!
            get_workbench().event_generate(
                "RemoteFileOperation", path=target_filename, operation="save"
            )
            get_workbench().event_generate("RemoteFilesChanged")
            return True
        else:
            messagebox.showerror(tr("Could not save"), tr("Back-end is not ready"))
            return False

    def ask_new_path(self, node=None):
        if node is None:
            node = choose_node_for_file_operations(self.winfo_toplevel(), tr("Where to save to?"))
        if not node:
            return None

        if node == "local":
            return self.ask_new_local_path()
        else:
            assert node == "remote"
            return self.ask_new_remote_path()

    def ask_new_remote_path(self):
        target_path = ask_backend_path(self.winfo_toplevel(), "save")
        if target_path:
            target_path = self._check_add_py_extension(target_path)
            return make_remote_path(target_path)
        else:
            return None

    def ask_new_local_path(self):
        if self._filename is None:
            initialdir = get_workbench().get_local_cwd()
            initialfile = None
        else:
            initialdir = os.path.dirname(self._filename)
            initialfile = os.path.basename(self._filename)

        # https://tcl.tk/man/tcl8.6/TkCmd/getOpenFile.htm
        type_var = tk.StringVar(value="")
        new_filename = asksaveasfilename(
            filetypes=_dialog_filetypes,
            defaultextension=None,
            initialdir=initialdir,
            initialfile=initialfile,
            parent=get_workbench(),
            typevariable=type_var,
        )
        logger.info("Save dialog returned %r with typevariable %r", new_filename, type_var.get())

        # Different tkinter versions may return different values
        if new_filename in ["", (), None]:
            return None

        if running_on_windows():
            # may have /-s instead of \-s and wrong case
            new_filename = os.path.join(
                normpath_with_actual_case(os.path.dirname(new_filename)),
                os.path.basename(new_filename),
            )

        if type_var.get() == PYTHON_FILES_STR or type_var.get() == "":
            new_filename = self._check_add_py_extension(
                new_filename, without_asking=type_var.get() == PYTHON_FILES_STR
            )

        if new_filename.endswith(".py"):
            base = os.path.basename(new_filename)
            mod_name = base[:-3].lower()
            if running_on_windows():
                mod_name = mod_name.lower()

            if mod_name in [
                "math",
                "turtle",
                "random",
                "statistics",
                "pygame",
                "matplotlib",
                "numpy",
            ]:
                # More proper name analysis will be performed by ProgramNamingAnalyzer
                if not tk.messagebox.askyesno(
                    tr("Potential problem"),
                    tr(
                        "If you name your script '%s', "
                        "you won't be able to import the library module named '%s'"
                    )
                    % (base, mod_name)
                    + ".\n\n"
                    + tr("Do you still want to use this name for your script?"),
                    master=self,
                ):
                    return self.ask_new_local_path()

        return new_filename

    def show(self):
        self.notebook.select(self)

    def update_appearance(self):
        self._code_view.set_gutter_visibility(
            get_workbench().get_option("view.show_line_numbers") or get_workbench().in_simple_mode()
        )
        self._code_view.set_line_length_margin(
            get_workbench().get_option("view.recommended_line_length")
        )
        self._code_view.text.update_tabs()
        self._code_view.text.event_generate("<<UpdateAppearance>>")
        self._code_view.grid_main_widgets()

    def _listen_debugger_progress(self, event):
        # Go read-only
        # TODO: check whether this module is active?
        self._code_view.text.set_read_only(True)

    def _listen_for_toplevel_response(self, event: ToplevelResponse) -> None:
        self._code_view.text.set_read_only(False)

    def _control_tab(self, event):
        if event.state & 1:  # shift was pressed
            direction = -1
        else:
            direction = 1
        self.notebook.select_next_prev_editor(direction)
        return "break"

    def _shift_control_tab(self, event):
        self.notebook.select_next_prev_editor(-1)
        return "break"

    def select_range(self, text_range):
        self._code_view.select_range(text_range)

    def select_line(self, lineno, col_offset=None):
        self._code_view.select_range(TextRange(lineno, 0, lineno + 1, 0))
        self.see_line(lineno)

        if col_offset is None:
            col_offset = 0

        self.get_text_widget().mark_set("insert", "%d.%d" % (lineno, col_offset))

    def see_line(self, lineno):
        # first see an earlier line in order to push target line downwards
        self._code_view.text.see("%d.0" % max(lineno - 4, 1))
        self._code_view.text.see("%d.0" % lineno)

    def focus_set(self):
        self._code_view.focus_set()

    def is_focused(self):
        return self.focus_displayof() == self._code_view.text

    def _on_text_modified(self, event):
        self.update_title()

    def update_title(self):
        try:
            self.notebook.update_editor_title(self)
        except Exception as e:
            logger.exception("Could not update editor title", exc_info=e)

    def _on_text_change(self, event):
        self.update_title()

    def destroy(self):
        get_workbench().unbind("DebuggerResponse", self._listen_debugger_progress)
        get_workbench().unbind("ToplevelResponse", self._listen_for_toplevel_response)
        ttk.Frame.destroy(self)
        get_workbench().event_generate(
            "EditorTextDestroyed", editor=self, text_widget=self.get_text_widget()
        )

    def _check_add_py_extension(self, path: str, without_asking: bool = False) -> str:
        assert path
        parts = re.split(r"[/\\]", path)
        name = parts[-1]
        if "." not in name:
            if without_asking or messagebox.askyesno(
                title=tr("Confirmation"),
                message=tr("Python files usually have .py extension.")
                        + "\n\n"
                        + tr("Did you mean '%s'?" % (name + ".py")),
                parent=self.winfo_toplevel(),
            ):
                return path + ".py"
            else:
                return path

        return path


class ACFileDialog(CommonDialog):
    def __init__(self, master):
        super().__init__(master=master)
        self.title("曙光云文件管理")
        self.geometry("800x600")
        self.button_frame = tk.Frame(self)
        self.button_frame.pack(side=tk.TOP)

        self.back_button = tk.Button(self.button_frame, text="后退", command=self.go_back)
        self.back_button.pack(side=tk.LEFT, anchor=tk.W)

        self.new_button = tk.Button(self.button_frame, text="新建目录", command=self.create_file)
        self.new_button.pack(side=tk.LEFT, padx=5)

        self.upload_button = tk.Button(self.button_frame, text="上传", command=self.upload_file)
        self.upload_button.pack(side=tk.LEFT, padx=5)

        self.upload_button = tk.Button(self.button_frame, text="刷新", command=self.refresh_list)
        self.upload_button.pack(side=tk.LEFT, padx=5)

        self.file_tree = ttk.Treeview(self)
        self.file_tree.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        self.file_info_frame = tk.Frame(self)
        self.file_info_frame.pack(side=tk.RIGHT, fill=tk.BOTH, expand=True)

        self.file_info_label = tk.Label(self.file_info_frame, text="文件信息：")
        self.file_info_label.pack()

        self.open_button = tk.Button(self.file_info_frame, text="打开", command=self.open_file)
        self.open_button.configure(state=tk.DISABLED)
        self.open_button.pack()

        closed_folder = os.path.join(os.path.dirname(__file__), "res", "closed-folder.gif")
        self.folder_close_icon = tk.PhotoImage(file=closed_folder)
        self.data = None
        self.default_text = None
        self.file_name = None
        self.current_path = None
        self.parent_path = None
        self.load_files()

    def load_files(self, folder_path=None):
        print(folder_path)
        self.file_tree.delete(*self.file_tree.get_children())
        token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
        enable_url = get_enable_url(token, "efileUrls")
        search_file = search_files(token, enable_url, 10, 0, None, folder_path)
        self.data = json.loads(search_file)
        self.current_path = self.data["data"]["path"]
        file_list = self.data["data"]["fileList"]
        children_list = self.data["data"]["children"]

        for child in children_list:
            self.file_tree.insert(
                "", tk.END, text=f" {child['label']}", values=child, image=self.folder_close_icon
            )
        for file in file_list:
            self.file_tree.insert("", tk.END, text=file["name"], values=file)

        self.file_tree.bind("<<TreeviewSelect>>", self.show_file_info)

    def show_file_info(self, event):
        selected_item = self.file_tree.focus()
        file_info_text = ""
        file = self.file_tree.item(selected_item)
        if file["image"]:
            if file["image"][0].startswith("pyimage1"):
                path = (
                    file["values"][0].split("'path': ")[1].split(",")[0].strip("'")
                )  # 使用字符串分割提取路径,去除路径中的引号
                file_info_text += f"文件夹名: {file['text']}\n"
                file_info_text += f"文件路径: {path}\n"
                self.open_button.configure(state=tk.NORMAL)
        else:
            file_info_text += f"文件夹名: {file['text']}\n"
            path = (
                file["values"][0].split("'path': ")[1].split(",")[0].strip("'")
            )  # 使用字符串分割提取路径,去除路径中的引号
            file_info_text += f"文件路径: {path}\n"
            owner = file["values"][0].split("'owner': ")[1].split(",")[0].strip("'")
            file_info_text += f"创建者: {owner}\n"
            size = file["values"][0].split("'size': ")[1].split(",")[0].strip("'")
            file_info_text += f"文件大小: {size} bytes\n"
            lastModifiedTime = (
                file["values"][0].split("'lastModifiedTime': ")[1].split(",")[0].strip("'")
            )
            file_info_text += f"最后修改时间: {lastModifiedTime}"
            print(file["values"])
            self.open_button.configure(state=tk.NORMAL)
            # self.open_button.configure(state=tk.DISABLED)

        self.file_info_label.configure(text=file_info_text)

    def open_file(self):
        selected_item = self.file_tree.focus()
        file = self.file_tree.item(selected_item)
        path = file["values"][0].split("'path': ")[1].split(",")[0].strip("'")
        if file["image"]:
            print(f"打开文件夹路径: {path}")
            self.parent_path = self.current_path
            self.load_files(folder_path=path)
        else:
            print(f"打开文件路径: {path}")
            token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
            enable_url = get_enable_url(token, "hpcUrls")
            file_content = view_files(token, enable_url, path)
            file_content_o = json.loads(file_content)
            print(file_content_o)
            if file_content_o["code"] == "0":
                if file_content_o["data"]["success"] == "true":
                    self.file_name = file["text"]
                    self.default_text = file_content_o["data"]["data"]
                    print(f"文本内容: {self.default_text}")
                    editor_window = tk.Toplevel(self)
                    text_area = ScrolledText(editor_window, wrap=tk.WORD)
                    text_area.insert("1.0", self.default_text)
                    text_area.pack(fill=tk.BOTH, expand=True)

                    def save_on_ctrl_s():
                        text_content = text_area.get("1.0", tk.END)
                        print(text_content)
                        try:
                            with open(self.file_name, "w") as f:
                                f.write(text_content)
                            efile_url = get_enable_url(token, "efileUrls")
                            print(os.path.dirname(path))
                            upload_file(
                                token, efile_url, os.path.dirname(path), "cover", self.file_name
                            )
                            self.default_text = text_content
                            tk.messagebox.showinfo("保存成功", "文本保存成功。")
                            editor_window.destroy()
                        except Exception as e:
                            tk.messagebox.showerror("保存失败", f"保存文本时出错: {str(e)}")

                        # 使用完毕后删除临时文件
                        try:
                            os.remove(self.file_name)
                        except Exception as e:
                            print(f"Failed to delete the temporary file: {str(e)}")

                    editor_window.bind("<Control-s>", save_on_ctrl_s)

                    save_button = tk.Button(editor_window, text="保存", command=save_on_ctrl_s)
                    save_button.pack()

                    def on_close():
                        current_text = text_area.get("1.0", tk.END).strip()
                        if current_text != self.default_text:
                            result = messagebox.askyesnocancel("关闭文本", "是否要在关闭前保存更改？")
                            if result is True:
                                save_on_ctrl_s()
                                editor_window.destroy()
                            elif result is False:
                                editor_window.destroy()
                        else:
                            editor_window.destroy()

                    editor_window.protocol("WM_DELETE_WINDOW", on_close)
                else:
                    # 显示提示信息
                    self.open_button.configure(state=tk.DISABLED)
                    messagebox.showinfo("提示", f"目前仅支持文本文件打开查看")

            else:
                # 显示提示信息
                messagebox.showinfo("提示", f"远程连接失败")

    def go_back(self):
        # 执行返回操作
        # 调用接口或编写逻辑以实现后退功能
        print("Go Back")
        self.load_files(folder_path=self.parent_path)

    def create_file(self):
        # 执行新建文件操作
        # 调用接口或编写逻辑以实现新建文件功能
        print("Create File")
        file_name = simpledialog.askstring("新建目录", "请输入文件名")
        if file_name:
            # 调用接口或编写逻辑以实现创建文件功能
            print(f"Creating File: {file_name}")
            print(f"current_path: {self.current_path}")
            token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
            enable_url = get_enable_url(token, "efileUrls")
            mkdir(token, enable_url, self.current_path + "/" + file_name, "false")
            # 显示提示信息
            messagebox.showinfo("提示", f"创建文件 {file_name} 完成")

    def refresh_list(self):
        # 执行刷新文件列表操作
        # 调用接口或编写逻辑以实现新建文件功能
        print("Refresh List")
        self.load_files(folder_path=self.current_path)

    def upload_file(self):
        # 执行上传文件操作
        # 调用接口或编写逻辑以实现上传文件功能
        print("Upload File")
        file_path = filedialog.askopenfilename()
        if file_path:
            # 调用接口或编写逻辑以实现上传文件功能
            print(f"Uploading File: {file_path}")
            token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
            enable_url = get_enable_url(token, "efileUrls")
            upload_file(token, enable_url, self.current_path, "uncover", file_path)


class JobSubmitDialog(CommonDialog):
    def __init__(self, master):
        super().__init__(master=master)

        self.title("提交作业")
        self.geometry("400x600")

        self.create_widgets()

    def create_widgets(self):
        # 创建下拉选择框
        label_submission = tk.Label(self, text="提交方式:")
        label_submission.pack()
        self.combo_submission = ttk.Combobox(self, values=["命令行方式"])
        self.combo_submission.pack()
        default_value = "命令行方式"
        self.combo_submission.current(self.combo_submission["values"].index(default_value))

        # 创建标签和输入框
        label_name = tk.Label(self, text="作业名称:")
        label_name.pack()
        self.entry_name = tk.Entry(self)
        self.entry_name.pack()

        # 创建选择文件目录按钮
        label_directory = tk.Label(self, text="工作目录:")
        label_directory.pack()
        self.entry_directory = tk.Entry(self)
        self.entry_directory.pack()
        # button_select_directory = tk.Button(self, text="选择文件目录", command=self.select_directory)
        # button_select_directory.pack()

        # 创建命令行方式的文本输入框
        label_command = tk.Label(self, text="命令行方式:")
        label_command.pack()
        self.text_command = tk.Text(self, height=10)
        self.text_command.pack()
        default_text = "sleep 5"
        self.text_command.insert("1.0", default_text)

        # 创建提交按钮
        button_submit = tk.Button(self, text="提交", command=self.submit_homework)
        button_submit.pack()

    def submit_homework(self):
        # 获取下拉选择框的值
        submission_method = self.combo_submission.get()

        # 获取文本输入框的值
        job_name = self.entry_name.get()
        working_directory = self.entry_directory.get()
        command_line = self.text_command.get("1.0", tk.END)

        # 打印提交的作业信息
        print("提交方式:", submission_method)
        print("作业名称:", job_name)
        print("工作目录:", working_directory)

        if submission_method == "命令行方式":
            print("命令行:", command_line)
            if not command_line.strip():
                # command_line为空
                messagebox.showerror(
                    "提交作业",
                    "命令行脚本不能为空\n\n",
                    master=get_workbench(),
                )
            else:
                if command_line:
                    # 在这里可以将作业提交到服务器或进行其他处理
                    # todo 曙光账号来源
                    token = get_tokens("acsx6nwsm6", "yubo@liuzhongxiu")
                    enable_url = get_enable_url(token, "hpcUrls")
                    job_cluster = get_job_cluster(token, enable_url)
                    job_queues = get_job_queues(token, enable_url, "acsx6nwsm6", job_cluster)
                    job_info = submit_job(
                        token,
                        enable_url,
                        job_cluster,
                        job_queues,
                        command_line,
                        job_name,
                        working_directory,
                    )
                    print("提交请求:", job_info)
                    # 提交成功提示窗口
                    messagebox.showinfo("提交成功", "作业已成功提交！\n" + job_info)
                    # 关闭弹出框
                    self.destroy()

    def select_directory(self):
        # 弹出选择文件夹对话框
        directory = filedialog.askdirectory()
        # 将选择的文件夹路径显示在输入框中
        self.entry_directory.delete(0, tk.END)
        self.entry_directory.insert(tk.END, directory)


class EditorNotebook(CustomNotebook):
    """
    Manages opened files / modules
    """

    def __init__(self, master):
        super().__init__(master)

        get_workbench().set_default("file.reopen_all_files", False)
        get_workbench().set_default("file.open_files", [])
        get_workbench().set_default("file.current_file", None)
        get_workbench().set_default("file.recent_files", [])
        get_workbench().set_default("view.highlight_current_line", False)
        get_workbench().set_default("view.show_line_numbers", True)
        get_workbench().set_default("view.recommended_line_length", 0)
        get_workbench().set_default("edit.indent_with_tabs", False)
        get_workbench().set_default("edit.auto_refresh_saved_files", True)
        get_workbench().set_default("file.make_saved_shebang_scripts_executable", True)

        self._recent_menu = tk.Menu(
            get_workbench().get_menu("file"), postcommand=self._update_recent_menu
        )
        self._init_commands()
        self.enable_traversal()

        # open files from last session
        """ TODO: they should go only to recent files
        for filename in prefs["open_files"].split(";"):
            if os.path.exists(filename):
                self._open_file(filename)
        """

        # should be in the end, so that it can be detected when
        # constructor hasn't completed yet
        self.preferred_size_in_pw = None
        self._checking_external_changes = False

        get_workbench().bind("WindowFocusIn", self.check_for_external_changes, True)
        get_workbench().bind("ToplevelResponse", self.check_for_external_changes, True)
        self.bind("<<NotebookTabChanged>>", self.on_tab_changed, True)

    def on_tab_changed(self, *args):
        # Required to avoid incorrect sizing of parent panes
        self.update_idletasks()

    def _init_commands(self):
        # TODO: do these commands have to be in EditorNotebook ??
        # Create a module level function install_editor_notebook ??
        # Maybe add them separately, when notebook has been installed ??

        get_workbench().add_command(
            "new_file",
            "file",
            tr("New"),
            self._cmd_new_file,
            caption=tr("New"),
            default_sequence=select_sequence("<Control-n>", "<Command-n>"),
            extra_sequences=["<Control-Greek_nu>"],
            group=10,
            image="new-file",
            include_in_toolbar=True,
        )

        get_workbench().add_command(
            "watch_video",
            "course",
            tr("课程视频"),
            self._cmd_watch_video,
            caption=tr("Video"),
            default_sequence=select_sequence("<Control-n>", "<Command-n>"),
            extra_sequences=["<Control-Greek_nu>"],
            group=10,
            image="video",
            include_in_toolbar=True,
        )

        get_workbench().add_command(
            "homework_and_exam",
            "course",
            "课程作业",
            self._cmd_homework_and_exam,
            caption=tr("Homework And Exam"),
            default_sequence=select_sequence("<Control-j>", "<Command-j>"),
            extra_sequences=["<Control-Greek_nu>"],
            group=10,
            image="homework 16x16",
            include_in_toolbar=True,
        )

        get_workbench().add_command(
            "job_submit",
            "AC",
            tr("曙光-文件提交"),
            self._cmd_job_submit,
            caption=tr("AC"),
            # default_sequence=select_sequence("<Control-[>", "<Command-[>"),
            # extra_sequences=["<Control-Greek_nu>"],
            group=10,
            image="ac-submit",
            include_in_toolbar=True,
        )

        get_workbench().add_command(
            "ac_file",
            "AC",
            tr("曙光-文件管理"),
            self._cmd_ac_file,
            caption=tr("ac_file_manage"),
            # default_sequence=select_sequence("<Control-'>", "<Command-'>"),
            # extra_sequences=["<Control-Greek_nu>"],
            group=10,
            image="ac-manager",
            include_in_toolbar=True,
        )

        get_workbench().add_command(
            "watch_work",
            "AC",
            "曙光-作业",
            self._cmd_watch_work,
            caption="ac_homework",
            # default_sequence=select_sequence("<Control-;>", "<Command-;>"),
            # extra_sequences=["<Control-Greek_nu>"],
            group=10,
            image="ac-homework",
            include_in_toolbar=True,
        )

        get_workbench().add_command(
            "recents", "file", tr("Recent files"), group=10, submenu=self._recent_menu
        )

        # http://stackoverflow.com/questions/22907200/remap-default-keybinding-in-tkinter
        get_workbench().bind_class("Text", "<Control-o>", self._control_o)
        get_workbench().bind_class("Text", "<Control-Greek_omicron>", self._control_o)
        rebind_control_a(get_workbench())

        get_workbench().add_command(
            "close_file",
            "file",
            tr("Close"),
            self._cmd_close_file,
            default_sequence=select_sequence("<Control-w>", "<Command-w>"),
            extra_sequences=["<Control-Greek_finalsmallsigma>"],
            tester=lambda: self.get_current_editor() is not None,
            group=10,
        )

        get_workbench().add_command(
            "close_files",
            "file",
            tr("Close all"),
            self.close_tabs,
            tester=lambda: self.get_current_editor() is not None,
            default_sequence=select_sequence("<Control-W>", "<Command-Alt-w>"),
            group=10,
        )

        get_workbench().add_command(
            "save_file",
            "file",
            tr("Save"),
            self._cmd_save_file,
            caption=tr("Save"),
            default_sequence=select_sequence("<Control-s>", "<Command-s>"),
            extra_sequences=["<Control-Greek_sigma>"],
            tester=self._cmd_save_file_enabled,
            group=10,
            image="save-file",
            include_in_toolbar=True,
        )

        get_workbench().add_command(
            "save_all_files",
            "file",
            tr("Save All files"),
            self._cmd_save_all_files,
            caption=tr("Save All files"),
            default_sequence=select_sequence("<Control-Alt-s>", "<Command-Alt-s>"),
            tester=self._cmd_save_all_files_enabled,
            group=10,
        )

        get_workbench().add_command(
            "save_file_as",
            "file",
            tr("Save as..."),
            self._cmd_save_file_as,
            default_sequence=select_sequence("<Control-Shift-S>", "<Command-Shift-S>"),
            extra_sequences=["<Control-Greek_SIGMA>"],
            tester=lambda: self.get_current_editor() is not None,
            group=10,
        )

        get_workbench().add_command(
            "save_copy",
            "file",
            tr("Save copy..."),
            self._cmd_save_copy,
            tester=lambda: self.get_current_editor() is not None,
            group=10,
        )

        get_workbench().add_command(
            "move_rename_file",
            "file",
            tr("Move / rename..."),
            self._cmd_move_rename_file,
            tester=self._cmd_move_rename_file_enabled,
            group=10,
        )

        get_workbench().add_command(
            "goto_source_line",
            "edit",
            tr("Go to line..."),
            self._cmd_goto_source_line,
            default_sequence=select_sequence("<Control-g>", "<Command-g>"),
            # tester=,
            # no global switch, or cross plugin switch?
            # todo use same as find and replace -> plugins/find_replace.py
            group=60,
        )

        get_workbench().createcommand("::tk::mac::OpenDocument", self._mac_open_document)

    def _cmd_homework_and_exam(self):
        ui_utils.show_dialog(homeworkAndExamDialog(get_workbench()))

    def load_startup_files(self):
        """
        这行代码是一个列表推导式，用于获取命令行参数中存在的文件的绝对路径。

        让我们逐个解析这段代码：

        sys.argv[1:]：sys.argv 是一个包含命令行参数的列表，其中第一个元素是脚本的名称，后续元素是传递给脚本的参数。sys.argv[1:] 表示从第二个参数开始（即索引为 1 的元素）直到最后一个参数，切片出来的部分由 : 表示。

        os.path.exists(name)：os.path.exists() 是一个函数，用于检查给定路径是否存在。在这里，name 是指从命令行参数中取得的每个文件名。

        os.path.abspath(name)：os.path.abspath() 是一个函数，用于获取给定路径的绝对路径。在这里，name 是每个存在的文件名。

        for name in sys.argv[1:] if os.path.exists(name)：这是一个列表推导式的语法结构。它遍历 sys.argv[1:] 中的每个参数，但只有当 os.path.exists(name) 返回 True（即文件存在）时才会执行后续操作。

        因此，这段代码的作用是对于命令行参数中存在的每个文件名，获取其绝对路径，并将这些绝对路径以列表的形式输出。换句话说，它过滤出命令行参数中真实存在的文件，并返回这些文件的绝对路径列表。

        """
        """If no filename was sent from command line
        then load previous files (if setting allows)"""

        cmd_line_filenames = [
            os.path.abspath(name) for name in sys.argv[1:] if os.path.exists(name)
        ]

        if len(cmd_line_filenames) > 0:
            filenames = cmd_line_filenames
        elif get_workbench().get_option("file.reopen_all_files"):
            filenames = get_workbench().get_option("file.open_files")
        elif get_workbench().get_option("file.current_file"):
            filenames = [get_workbench().get_option("file.current_file")]
        else:
            filenames = []

        if len(filenames) > 0:
            for filename in filenames:
                if os.path.exists(filename):
                    self.show_file(filename)

            cur_file = get_workbench().get_option("file.current_file")
            # choose correct active file
            if len(cmd_line_filenames) > 0:
                self.show_file(cmd_line_filenames[0])
            elif cur_file and os.path.exists(cur_file):
                self.show_file(cur_file)
            else:
                self._cmd_new_file()
        else:
            self._cmd_new_file()

    def save_all_named_editors(self):
        all_saved = True
        for editor in self.winfo_children():
            if editor.get_filename() and editor.is_modified():
                success = editor.save_file()
                all_saved = all_saved and success

        return all_saved

    def remember_recent_file(self, filename):
        recents = get_workbench().get_option("file.recent_files")
        if filename in recents:
            recents.remove(filename)
        recents.insert(0, filename)
        relevant_recents = [name for name in recents if os.path.exists(name)][:15]
        get_workbench().set_option("file.recent_files", relevant_recents)
        self._update_recent_menu()

    def _update_recent_menu(self):
        recents = get_workbench().get_option("file.recent_files")
        relevant_recents = [
            path for path in recents if os.path.exists(path) and not self.file_is_opened(path)
        ]
        self._recent_menu.delete(0, "end")
        for path in relevant_recents:
            def load(path=path):
                self.show_file(path)

            self._recent_menu.insert_command("end", label=path, command=load)

    def remember_open_files(self):
        if (
            self.get_current_editor() is not None
            and self.get_current_editor().get_filename() is not None
        ):
            current_file = self.get_current_editor().get_filename()
        else:
            current_file = None

        get_workbench().set_option("file.current_file", current_file)

        open_files = [
            editor.get_filename() for editor in self.winfo_children() if editor.get_filename()
        ]
        get_workbench().set_option("file.open_files", open_files)

    # 获取视频
    def _cmd_watch_video(self):
        ui_utils.show_dialog(VideoDialog(get_workbench()))

    def _cmd_watch_work(self):
        ui_utils.show_dialog(WorkDialog(get_workbench()))

    def _cmd_job_submit(self):
        ui_utils.show_dialog(JobSubmitDialog(get_workbench()))

    def _cmd_ac_file(self):
        ui_utils.show_dialog(ACFileDialog(get_workbench()))

    def _cmd_new_file(self):
        self.open_new_file()

    def open_new_file(self, path=None, remote=False):
        new_editor = Editor(self)
        get_workbench().event_generate("NewFile", editor=new_editor)
        if path:
            if remote:
                new_editor.set_filename(make_remote_path(path))
            else:
                new_editor.set_filename(path)
        self.add(new_editor, text=new_editor.get_title())
        self.select(new_editor)
        new_editor.focus_set()

    def _cmd_open_file(self):
        node = choose_node_for_file_operations(self.winfo_toplevel(), "Where to open from?")
        if not node:
            return

        if node == "local":
            initialdir = get_workbench().get_local_cwd()
            if (
                self.get_current_editor() is not None
                and self.get_current_editor().get_filename() is not None
            ):
                initialdir = os.path.dirname(self.get_current_editor().get_filename())
            path = askopenfilename(
                filetypes=_dialog_filetypes, initialdir=initialdir, parent=get_workbench()
            )
        else:
            assert node == "remote"
            target_path = ask_backend_path(self.winfo_toplevel(), "open")
            if not target_path:
                return

            path = make_remote_path(target_path)

        if path:
            # self.close_single_untitled_unmodified_editor()
            self.show_file(path, propose_dialog=False)

    def _control_o(self, event):
        # http://stackoverflow.com/questions/22907200/remap-default-keybinding-in-tkinter
        self._cmd_open_file()
        return "break"

    def _close_files(self, except_index=None):
        for tab_index in reversed(range(len(self.winfo_children()))):
            if except_index is not None and tab_index == except_index:
                continue
            else:
                editor = self.get_child_by_index(tab_index)
                if self.check_allow_closing(editor):
                    self.forget(editor)
                    editor.destroy()

    def _cmd_close_file(self):
        self.close_tab(self.index(self.select()))

    def close_tab(self, index_or_tab: Union[int, CustomNotebookTab]):
        if isinstance(index_or_tab, int):
            page = self.pages[index_or_tab]
        else:
            page = self.get_page_by_tab(index_or_tab)

        assert isinstance(page.content, Editor)
        self.close_editor(page.content)

    def close_editor(self, editor, force=False):
        if not force and not self.check_allow_closing(editor):
            return
        self.forget(editor)
        editor.destroy()

    def _cmd_save_file(self):
        if self.get_current_editor():
            self.get_current_editor().save_file()
            self.update_editor_title(self.get_current_editor())

    def _cmd_save_file_enabled(self):
        return self.get_current_editor() and self.get_current_editor().save_file_enabled()

    def _cmd_save_all_files(self):
        for editor in self.get_all_editors():
            if editor.save_file_enabled() == True:
                editor.save_file()
                self.update_editor_title(editor)

    def _cmd_save_all_files_enabled(self):
        for editor in self.get_all_editors():
            if editor.save_file_enabled() == True:
                return True
        return False

    def _cmd_save_file_as(self, node=None):
        if not self.get_current_editor():
            return

        self.get_current_editor().save_file(ask_filename=True, node=node)
        self.update_editor_title(self.get_current_editor())
        get_workbench().update_title()

    def _cmd_save_copy(self):
        if not self.get_current_editor():
            return

        self.get_current_editor().save_file(ask_filename=True, save_copy=True)
        self.update_editor_title(self.get_current_editor())

    def _cmd_save_file_as_enabled(self):
        return self.get_current_editor() is not None

    def _cmd_move_rename_file(self):
        editor = self.get_current_editor()
        old_filename = editor.get_filename()
        assert old_filename is not None

        if is_remote_path(old_filename):
            node = "remote"
        else:
            node = "local"

        self._cmd_save_file_as(node=node)

        if editor.get_filename() != old_filename:
            if is_remote_path(old_filename):
                remote_path = extract_target_path(old_filename)
                get_runner().send_command_and_wait(
                    InlineCommand(
                        "delete", paths=[remote_path], description=tr("Deleting" + remote_path)
                    ),
                    dialog_title=tr("Deleting"),
                )
                get_workbench().event_generate(
                    "RemoteFileOperation", path=remote_path, operation="delete"
                )
            else:
                os.remove(old_filename)
                get_workbench().event_generate(
                    "LocalFileOperation", path=old_filename, operation="delete"
                )

    def _cmd_move_rename_file_enabled(self):
        return self.get_current_editor() and self.get_current_editor().get_filename() is not None

    def close_single_untitled_unmodified_editor(self):
        editors = self.winfo_children()
        if len(editors) == 1 and not editors[0].is_modified() and not editors[0].get_filename():
            self._cmd_close_file()

    def _cmd_goto_source_line(self):
        editor = self.get_current_editor()
        if editor:
            line_no = simpledialog.askinteger(tr("Go to line"), tr("Line number"))
            if line_no:
                editor.select_line(line_no)

    def _mac_open_document(self, *args):
        for arg in args:
            if isinstance(arg, str) and os.path.exists(arg):
                self.show_file(arg)
        get_workbench().become_active_window()

    def get_current_editor(self) -> Optional[Editor]:
        return self.get_current_child()

    def get_current_editor_content(self):
        editor = self.get_current_editor()
        if editor is None:
            return None
        else:
            return editor.get_content()

    def get_all_editors(self):
        # When workspace is closing, self.winfo_children()
        # may return an unexplainable tkinter.Frame
        return [child for child in self.winfo_children() if isinstance(child, Editor)]

    def select_next_prev_editor(self, direction):
        cur_index = self.index(self.select())
        next_index = (cur_index + direction) % len(self.tabs())
        self.select(self.get_child_by_index(next_index))

    def file_is_opened(self, path):
        for editor in self.get_all_editors():
            if editor.get_filename() and is_same_path(path, editor.get_filename()):
                return True

        return False

    def show_file(self, filename, text_range=None, set_focus=True, propose_dialog=True):
        # self.close_single_untitled_unmodified_editor()
        try:
            editor = self.get_editor(filename, True)
        except PermissionError:
            logger.exception("Loading " + filename)
            msg = tr("Got permission error when trying to load\n%s") % (filename,)
            if running_on_mac_os() and propose_dialog:
                msg += "\n\n" + tr("Try opening it with File => Open.")

            messagebox.showerror(tr("Permission error"), msg, master=self)
            return None

        if editor is None:
            return

        self.select(editor)
        if set_focus:
            editor.focus_set()

        if text_range is not None:
            editor.select_range(text_range)

        return editor

    def show_remote_file(self, target_filename):
        if not get_runner().ready_for_remote_file_operations(show_message=True):
            return None
        else:
            return self.show_file(make_remote_path(target_filename))

    def show_file_at_line(self, filename, lineno, col_offset=None):
        editor = self.show_file(filename)
        editor.select_line(lineno, col_offset)

    def update_appearance(self):
        for editor in self.winfo_children():
            editor.update_appearance()

    def update_editor_title(self, editor, title=None):
        if title is None:
            title = editor.get_title()
        try:
            self.tab(editor, text=title)
        except TclError:
            pass

        try:
            self.indicate_modification()
        except Exception:
            logger.exception("Could not update modification indication")

    def indicate_modification(self):
        if not running_on_mac_os():
            return

        atts = self.winfo_toplevel().wm_attributes()
        if "-modified" in atts:
            i = atts.index("-modified")
            mod = atts[i: i + 2]
            rest = atts[:i] + atts[i + 2:]
        else:
            mod = ()
            rest = atts

        for editor in self.get_all_editors():
            if editor.is_modified():
                if mod != ("-modified", 1):
                    self.winfo_toplevel().wm_attributes(*(rest + ("-modified", 1)))
                break
        else:
            if mod == ("-modified", 1):
                self.winfo_toplevel().wm_attributes(*(rest + ("-modified", 0)))

    def _open_file(self, filename):
        editor = Editor(self)
        if editor._load_file(filename):
            self.add(editor, text=editor.get_title())
            return editor
        else:
            editor.destroy()
            return None

    def get_editor(self, filename_or_id, open_when_necessary=False):
        if os.path.isfile(filename_or_id):
            filename_or_id = normpath_with_actual_case(os.path.abspath(filename_or_id))

        for child in self.winfo_children():
            assert isinstance(child, Editor)
            child_identifier = child.get_identifier()
            if child_identifier == filename_or_id:
                return child

        if open_when_necessary:
            return self._open_file(filename_or_id)
        else:
            return None

    def check_allow_closing(self, editor=None):
        if not editor:
            modified_editors = [e for e in self.winfo_children() if e.is_modified()]
        else:
            if not editor.is_modified():
                return True
            else:
                modified_editors = [editor]
        if len(modified_editors) == 0:
            return True

        message = tr("Do you want to save files before closing?")
        if editor:
            message = tr("Do you want to save file before closing?")

        confirm = messagebox.askyesnocancel(
            title=tr("Save On Close"), message=message, default=messagebox.YES, master=self
        )

        if confirm:
            for editor_ in modified_editors:
                assert isinstance(editor_, Editor)
                if editor_.get_filename(True):
                    if not editor_.save_file():
                        return False
                else:
                    return False
            return True

        elif confirm is None:
            return False
        else:
            return True

    def check_for_external_changes(self, event=None):
        if self._checking_external_changes:
            # otherwise the method will be re-entered when focus
            # changes because of a confirmation message box
            return

        self._checking_external_changes = True
        try:
            for editor in self.get_all_editors():
                editor.check_for_external_changes()
        finally:
            self._checking_external_changes = False


def get_current_breakpoints():
    result = {}

    for editor in get_workbench().get_editor_notebook().get_all_editors():
        filename = editor.get_filename()
        if filename:
            linenos = editor.get_code_view().get_breakpoint_line_numbers()
            if linenos:
                result[filename] = linenos

    return result


def get_saved_current_script_filename(force=True):
    editor = get_workbench().get_editor_notebook().get_current_editor()
    if not editor:
        return None

    filename = editor.get_filename(force)
    if not filename:
        return None

    if editor.is_modified():
        filename = editor.save_file()

    return filename


def get_target_dirname_from_editor_filename(s):
    if is_local_path(s):
        return os.path.dirname(s)
    else:
        return universal_dirname(extract_target_path(s))


def extract_target_path(s):
    assert is_remote_path(s)
    return s[s.find(REMOTE_PATH_MARKER) + len(REMOTE_PATH_MARKER):]


def make_remote_path(target_path):
    return get_runner().get_node_label() + REMOTE_PATH_MARKER + target_path
